//-----------------------------------------------------------------------------
//  INFO (State definitions)
//-----------------------------------------------------------------------------
//
//  DEH_EDGE  Copyright (C) 2004-2005  The EDGE Team
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License (in COPYING.txt) for more details.
//
//------------------------------------------------------------------------
//
//  DEH_EDGE is based on:
//
//  +  DeHackEd source code, by Greg Lewis.
//  -  DOOM source code (C) 1993-1996 id Software, Inc.
//  -  Linux DOOM Hack Editor, by Sam Lantinga.
//  -  PrBoom's DEH/BEX code, by Ty Halderman, TeamTNT.
//
//------------------------------------------------------------------------

#include "i_defs.h"
#include "info.h"

#include "dh_plugin.h"
#include "mobj.h"


namespace Deh_Edge
{

#define DEBUG_RANGES  0


state_t	states[NUMSTATES_BEX] =
{
	// Weapons...
    {SPR_TROO,0,-1,A_NULL,S_NULL,0,0},	// S_NULL
    {SPR_SHTG,4,0,A_Light0,S_NULL,0,0},	// S_LIGHTDONE
    {SPR_PUNG,0,1,A_WeaponReady,S_PUNCH,0,0},	// S_PUNCH
    {SPR_PUNG,0,1,A_Lower,S_PUNCHDOWN,0,0},	// S_PUNCHDOWN
    {SPR_PUNG,0,1,A_Raise,S_PUNCHUP,0,0},	// S_PUNCHUP
    {SPR_PUNG,1,4,A_NULL,S_PUNCH2,0,0},		// S_PUNCH1
    {SPR_PUNG,2,4,A_Punch,S_PUNCH3,0,0},	// S_PUNCH2
    {SPR_PUNG,3,5,A_NULL,S_PUNCH4,0,0},		// S_PUNCH3
    {SPR_PUNG,2,4,A_NULL,S_PUNCH5,0,0},		// S_PUNCH4
    {SPR_PUNG,1,5,A_ReFire,S_PUNCH,0,0},	// S_PUNCH5
    {SPR_PISG,0,1,A_WeaponReady,S_PISTOL,0,0},// S_PISTOL
    {SPR_PISG,0,1,A_Lower,S_PISTOLDOWN,0,0},	// S_PISTOLDOWN
    {SPR_PISG,0,1,A_Raise,S_PISTOLUP,0,0},	// S_PISTOLUP
    {SPR_PISG,0,4,A_NULL,S_PISTOL2,0,0},	// S_PISTOL1
    {SPR_PISG,1,6,A_FirePistol,S_PISTOL3,0,0},// S_PISTOL2
    {SPR_PISG,2,4,A_NULL,S_PISTOL4,0,0},	// S_PISTOL3
    {SPR_PISG,1,5,A_ReFire,S_PISTOL,0,0},	// S_PISTOL4
    {SPR_PISF,32768,7,A_Light1,S_LIGHTDONE,0,0},	// S_PISTOLFLASH
    {SPR_SHTG,0,1,A_WeaponReady,S_SGUN,0,0},	// S_SGUN
    {SPR_SHTG,0,1,A_Lower,S_SGUNDOWN,0,0},	// S_SGUNDOWN
    {SPR_SHTG,0,1,A_Raise,S_SGUNUP,0,0},	// S_SGUNUP
    {SPR_SHTG,0,3,A_NULL,S_SGUN2,0,0},	// S_SGUN1
    {SPR_SHTG,0,7,A_FireShotgun,S_SGUN3,0,0},	// S_SGUN2
    {SPR_SHTG,1,5,A_NULL,S_SGUN4,0,0},	// S_SGUN3
    {SPR_SHTG,2,5,A_NULL,S_SGUN5,0,0},	// S_SGUN4
    {SPR_SHTG,3,4,A_NULL,S_SGUN6,0,0},	// S_SGUN5
    {SPR_SHTG,2,5,A_NULL,S_SGUN7,0,0},	// S_SGUN6
    {SPR_SHTG,1,5,A_NULL,S_SGUN8,0,0},	// S_SGUN7
    {SPR_SHTG,0,3,A_NULL,S_SGUN9,0,0},	// S_SGUN8
    {SPR_SHTG,0,7,A_ReFire,S_SGUN,0,0},	// S_SGUN9
    {SPR_SHTF,32768,4,A_Light1,S_SGUNFLASH2,0,0},	// S_SGUNFLASH1
    {SPR_SHTF,32769,3,A_Light2,S_LIGHTDONE,0,0},	// S_SGUNFLASH2
    {SPR_SHT2,0,1,A_WeaponReady,S_DSGUN,0,0},	// S_DSGUN
    {SPR_SHT2,0,1,A_Lower,S_DSGUNDOWN,0,0},	// S_DSGUNDOWN
    {SPR_SHT2,0,1,A_Raise,S_DSGUNUP,0,0},	// S_DSGUNUP
    {SPR_SHT2,0,3,A_NULL,S_DSGUN2,0,0},	// S_DSGUN1
    {SPR_SHT2,0,7,A_FireShotgun2,S_DSGUN3,0,0},	// S_DSGUN2
    {SPR_SHT2,1,7,A_NULL,S_DSGUN4,0,0},	// S_DSGUN3
    {SPR_SHT2,2,7,A_CheckReload,S_DSGUN5,0,0},	// S_DSGUN4
    {SPR_SHT2,3,7,A_OpenShotgun2,S_DSGUN6,0,0},	// S_DSGUN5
    {SPR_SHT2,4,7,A_NULL,S_DSGUN7,0,0},	// S_DSGUN6
    {SPR_SHT2,5,7,A_LoadShotgun2,S_DSGUN8,0,0},	// S_DSGUN7
    {SPR_SHT2,6,6,A_NULL,S_DSGUN9,0,0},	// S_DSGUN8
    {SPR_SHT2,7,6,A_CloseShotgun2,S_DSGUN10,0,0},	// S_DSGUN9
    {SPR_SHT2,0,5,A_ReFire,S_DSGUN,0,0},	// S_DSGUN10
    {SPR_SHT2,1,7,A_NULL,S_DSNR2,0,0},	// S_DSNR1
    {SPR_SHT2,0,3,A_NULL,S_DSGUNDOWN,0,0},	// S_DSNR2
    {SPR_SHT2,32776,5,A_Light1,S_DSGUNFLASH2,0,0},	// S_DSGUNFLASH1
    {SPR_SHT2,32777,4,A_Light2,S_LIGHTDONE,0,0},	// S_DSGUNFLASH2
    {SPR_CHGG,0,1,A_WeaponReady,S_CHAIN,0,0},	// S_CHAIN
    {SPR_CHGG,0,1,A_Lower,S_CHAINDOWN,0,0},	// S_CHAINDOWN
    {SPR_CHGG,0,1,A_Raise,S_CHAINUP,0,0},	// S_CHAINUP
    {SPR_CHGG,0,4,A_FireCGun,S_CHAIN2,0,0},	// S_CHAIN1
    {SPR_CHGG,1,4,A_FireCGun,S_CHAIN3,0,0},	// S_CHAIN2
    {SPR_CHGG,1,0,A_ReFire,S_CHAIN,0,0},	// S_CHAIN3
    {SPR_CHGF,32768,5,A_Light1,S_LIGHTDONE,0,0},	// S_CHAINFLASH1
    {SPR_CHGF,32769,5,A_Light2,S_LIGHTDONE,0,0},	// S_CHAINFLASH2
    {SPR_MISG,0,1,A_WeaponReady,S_MISSILE,0,0},	// S_MISSILE
    {SPR_MISG,0,1,A_Lower,S_MISSILEDOWN,0,0},	// S_MISSILEDOWN
    {SPR_MISG,0,1,A_Raise,S_MISSILEUP,0,0},	// S_MISSILEUP
    {SPR_MISG,1,8,A_GunFlash,S_MISSILE2,0,0},	// S_MISSILE1
    {SPR_MISG,1,12,A_FireMissile,S_MISSILE3,0,0},	// S_MISSILE2
    {SPR_MISG,1,0,A_ReFire,S_MISSILE,0,0},	// S_MISSILE3
    {SPR_MISF,32768,3,A_Light1,S_MISSILEFLASH2,0,0},	// S_MISSILEFLASH1
    {SPR_MISF,32769,4,A_NULL,S_MISSILEFLASH3,0,0},	// S_MISSILEFLASH2
    {SPR_MISF,32770,4,A_Light2,S_MISSILEFLASH4,0,0},	// S_MISSILEFLASH3
    {SPR_MISF,32771,4,A_Light2,S_LIGHTDONE,0,0},	// S_MISSILEFLASH4
    {SPR_SAWG,2,4,A_WeaponReady,S_SAWB,0,0},	// S_SAW
    {SPR_SAWG,3,4,A_WeaponReady,S_SAW,0,0},	// S_SAWB
    {SPR_SAWG,2,1,A_Lower,S_SAWDOWN,0,0},	// S_SAWDOWN
    {SPR_SAWG,2,1,A_Raise,S_SAWUP,0,0},	// S_SAWUP
    {SPR_SAWG,0,4,A_Saw,S_SAW2,0,0},	// S_SAW1
    {SPR_SAWG,1,4,A_Saw,S_SAW3,0,0},	// S_SAW2
    {SPR_SAWG,1,0,A_ReFire,S_SAW,0,0},	// S_SAW3
    {SPR_PLSG,0,1,A_WeaponReady,S_PLASMA,0,0},	// S_PLASMA
    {SPR_PLSG,0,1,A_Lower,S_PLASMADOWN,0,0},	// S_PLASMADOWN
    {SPR_PLSG,0,1,A_Raise,S_PLASMAUP,0,0},	// S_PLASMAUP
    {SPR_PLSG,0,3,A_FirePlasma,S_PLASMA2,0,0},	// S_PLASMA1
    {SPR_PLSG,1,20,A_ReFire,S_PLASMA,0,0},	// S_PLASMA2
    {SPR_PLSF,32768,4,A_Light1,S_LIGHTDONE,0,0},	// S_PLASMAFLASH1
    {SPR_PLSF,32769,4,A_Light1,S_LIGHTDONE,0,0},	// S_PLASMAFLASH2
    {SPR_BFGG,0,1,A_WeaponReady,S_BFG,0,0},	// S_BFG
    {SPR_BFGG,0,1,A_Lower,S_BFGDOWN,0,0},	// S_BFGDOWN
    {SPR_BFGG,0,1,A_Raise,S_BFGUP,0,0},	// S_BFGUP
    {SPR_BFGG,0,20,A_BFGsound,S_BFG2,0,0},	// S_BFG1
    {SPR_BFGG,1,10,A_GunFlash,S_BFG3,0,0},	// S_BFG2
    {SPR_BFGG,1,10,A_FireBFG,S_BFG4,0,0},	// S_BFG3
    {SPR_BFGG,1,20,A_ReFire,S_BFG,0,0},	// S_BFG4
    {SPR_BFGF,32768,11,A_Light1,S_BFGFLASH2,0,0},	// S_BFGFLASH1
    {SPR_BFGF,32769,6,A_Light2,S_LIGHTDONE,0,0},	// S_BFGFLASH2

	// Things...
    {SPR_BLUD,2,8,A_NULL,S_BLOOD2,0,0},	// S_BLOOD1
    {SPR_BLUD,1,8,A_NULL,S_BLOOD3,0,0},	// S_BLOOD2
    {SPR_BLUD,0,8,A_NULL,S_NULL,0,0},	// S_BLOOD3
    {SPR_PUFF,32768,4,A_NULL,S_PUFF2,0,0},	// S_PUFF1
    {SPR_PUFF,1,4,A_NULL,S_PUFF3,0,0},	// S_PUFF2
    {SPR_PUFF,2,4,A_NULL,S_PUFF4,0,0},	// S_PUFF3
    {SPR_PUFF,3,4,A_NULL,S_NULL,0,0},	// S_PUFF4
    {SPR_BAL1,32768,4,A_NULL,S_TBALL2,0,0},	// S_TBALL1
    {SPR_BAL1,32769,4,A_NULL,S_TBALL1,0,0},	// S_TBALL2
    {SPR_BAL1,32770,6,A_NULL,S_TBALLX2,0,0},	// S_TBALLX1
    {SPR_BAL1,32771,6,A_NULL,S_TBALLX3,0,0},	// S_TBALLX2
    {SPR_BAL1,32772,6,A_NULL,S_NULL,0,0},	// S_TBALLX3
    {SPR_BAL2,32768,4,A_NULL,S_RBALL2,0,0},	// S_RBALL1
    {SPR_BAL2,32769,4,A_NULL,S_RBALL1,0,0},	// S_RBALL2
    {SPR_BAL2,32770,6,A_NULL,S_RBALLX2,0,0},	// S_RBALLX1
    {SPR_BAL2,32771,6,A_NULL,S_RBALLX3,0,0},	// S_RBALLX2
    {SPR_BAL2,32772,6,A_NULL,S_NULL,0,0},	// S_RBALLX3
    {SPR_PLSS,32768,6,A_NULL,S_PLASBALL2,0,0},	// S_PLASBALL
    {SPR_PLSS,32769,6,A_NULL,S_PLASBALL,0,0},	// S_PLASBALL2
    {SPR_PLSE,32768,4,A_NULL,S_PLASEXP2,0,0},	// S_PLASEXP
    {SPR_PLSE,32769,4,A_NULL,S_PLASEXP3,0,0},	// S_PLASEXP2
    {SPR_PLSE,32770,4,A_NULL,S_PLASEXP4,0,0},	// S_PLASEXP3
    {SPR_PLSE,32771,4,A_NULL,S_PLASEXP5,0,0},	// S_PLASEXP4
    {SPR_PLSE,32772,4,A_NULL,S_NULL,0,0},	// S_PLASEXP5
    {SPR_MISL,32768,1,A_NULL,S_ROCKET,0,0},	// S_ROCKET
    {SPR_BFS1,32768,4,A_NULL,S_BFGSHOT2,0,0},	// S_BFGSHOT
    {SPR_BFS1,32769,4,A_NULL,S_BFGSHOT,0,0},	// S_BFGSHOT2
    {SPR_BFE1,32768,8,A_NULL,S_BFGLAND2,0,0},	// S_BFGLAND
    {SPR_BFE1,32769,8,A_NULL,S_BFGLAND3,0,0},	// S_BFGLAND2
    {SPR_BFE1,32770,8,A_BFGSpray,S_BFGLAND4,0,0},	// S_BFGLAND3
    {SPR_BFE1,32771,8,A_NULL,S_BFGLAND5,0,0},	// S_BFGLAND4
    {SPR_BFE1,32772,8,A_NULL,S_BFGLAND6,0,0},	// S_BFGLAND5
    {SPR_BFE1,32773,8,A_NULL,S_NULL,0,0},	// S_BFGLAND6
    {SPR_BFE2,32768,8,A_NULL,S_BFGEXP2,0,0},	// S_BFGEXP
    {SPR_BFE2,32769,8,A_NULL,S_BFGEXP3,0,0},	// S_BFGEXP2
    {SPR_BFE2,32770,8,A_NULL,S_BFGEXP4,0,0},	// S_BFGEXP3
    {SPR_BFE2,32771,8,A_NULL,S_NULL,0,0},	// S_BFGEXP4
    {SPR_MISL,32769,8,A_Explode,S_EXPLODE2,0,0},	// S_EXPLODE1
    {SPR_MISL,32770,6,A_NULL,S_EXPLODE3,0,0},	// S_EXPLODE2
    {SPR_MISL,32771,4,A_NULL,S_NULL,0,0},	// S_EXPLODE3
    {SPR_TFOG,32768,6,A_SpawnSound,S_TFOG01,0,0}, // S_TFOG  /* MOD */ 
    {SPR_TFOG,32769,6,A_NULL,S_TFOG02,0,0},	// S_TFOG01
    {SPR_TFOG,32768,6,A_NULL,S_TFOG2,0,0},	// S_TFOG02
    {SPR_TFOG,32769,6,A_NULL,S_TFOG3,0,0},	// S_TFOG2
    {SPR_TFOG,32770,6,A_NULL,S_TFOG4,0,0},	// S_TFOG3
    {SPR_TFOG,32771,6,A_NULL,S_TFOG5,0,0},	// S_TFOG4
    {SPR_TFOG,32772,6,A_NULL,S_TFOG6,0,0},	// S_TFOG5
    {SPR_TFOG,32773,6,A_NULL,S_TFOG7,0,0},	// S_TFOG6
    {SPR_TFOG,32774,6,A_NULL,S_TFOG8,0,0},	// S_TFOG7
    {SPR_TFOG,32775,6,A_NULL,S_TFOG9,0,0},	// S_TFOG8
    {SPR_TFOG,32776,6,A_NULL,S_TFOG10,0,0},	// S_TFOG9
    {SPR_TFOG,32777,6,A_NULL,S_NULL,0,0},	// S_TFOG10
    {SPR_IFOG,32768,6,A_NULL,S_IFOG01,0,0},	// S_IFOG
    {SPR_IFOG,32769,6,A_NULL,S_IFOG02,0,0},	// S_IFOG01
    {SPR_IFOG,32768,6,A_NULL,S_IFOG2,0,0},	// S_IFOG02
    {SPR_IFOG,32769,6,A_NULL,S_IFOG3,0,0},	// S_IFOG2
    {SPR_IFOG,32770,6,A_NULL,S_IFOG4,0,0},	// S_IFOG3
    {SPR_IFOG,32771,6,A_NULL,S_IFOG5,0,0},	// S_IFOG4
    {SPR_IFOG,32772,6,A_NULL,S_NULL,0,0},	// S_IFOG5
    {SPR_PLAY,0,-1,A_NULL,S_NULL,0,0},	// S_PLAY
    {SPR_PLAY,0,4,A_NULL,S_PLAY_RUN2,0,0},	// S_PLAY_RUN1
    {SPR_PLAY,1,4,A_NULL,S_PLAY_RUN3,0,0},	// S_PLAY_RUN2
    {SPR_PLAY,2,4,A_NULL,S_PLAY_RUN4,0,0},	// S_PLAY_RUN3
    {SPR_PLAY,3,4,A_NULL,S_PLAY_RUN1,0,0},	// S_PLAY_RUN4
    {SPR_PLAY,4,12,A_NULL,S_PLAY,0,0},	// S_PLAY_ATK1
    {SPR_PLAY,32773,6,A_NULL,S_PLAY_ATK1,0,0},	// S_PLAY_ATK2
    {SPR_PLAY,6,4,A_NULL,S_PLAY_PAIN2,0,0},	// S_PLAY_PAIN
    {SPR_PLAY,6,4,A_Pain,S_PLAY,0,0},	// S_PLAY_PAIN2
    {SPR_PLAY,7,10,A_NULL,S_PLAY_DIE2,0,0},	// S_PLAY_DIE1
    {SPR_PLAY,8,10,A_PlayerScream,S_PLAY_DIE3,0,0},	// S_PLAY_DIE2
    {SPR_PLAY,9,10,A_Fall,S_PLAY_DIE4,0,0},	// S_PLAY_DIE3
    {SPR_PLAY,10,10,A_NULL,S_PLAY_DIE5,0,0},	// S_PLAY_DIE4
    {SPR_PLAY,11,10,A_NULL,S_PLAY_DIE6,0,0},	// S_PLAY_DIE5
    {SPR_PLAY,12,10,A_NULL,S_PLAY_DIE7,0,0},	// S_PLAY_DIE6
    {SPR_PLAY,13,-1,A_NULL,S_NULL,0,0},	// S_PLAY_DIE7
    {SPR_PLAY,14,5,A_NULL,S_PLAY_XDIE2,0,0},	// S_PLAY_XDIE1
    {SPR_PLAY,15,5,A_XScream,S_PLAY_XDIE3,0,0},	// S_PLAY_XDIE2
    {SPR_PLAY,16,5,A_Fall,S_PLAY_XDIE4,0,0},	// S_PLAY_XDIE3
    {SPR_PLAY,17,5,A_NULL,S_PLAY_XDIE5,0,0},	// S_PLAY_XDIE4
    {SPR_PLAY,18,5,A_NULL,S_PLAY_XDIE6,0,0},	// S_PLAY_XDIE5
    {SPR_PLAY,19,5,A_NULL,S_PLAY_XDIE7,0,0},	// S_PLAY_XDIE6
    {SPR_PLAY,20,5,A_NULL,S_PLAY_XDIE8,0,0},	// S_PLAY_XDIE7
    {SPR_PLAY,21,5,A_NULL,S_PLAY_XDIE9,0,0},	// S_PLAY_XDIE8
    {SPR_PLAY,22,-1,A_NULL,S_NULL,0,0},	// S_PLAY_XDIE9
    {SPR_POSS,0,10,A_Look,S_POSS_STND2,0,0},	// S_POSS_STND
    {SPR_POSS,1,10,A_Look,S_POSS_STND,0,0},	// S_POSS_STND2
    {SPR_POSS,0,4,A_Chase,S_POSS_RUN2,0,0},	// S_POSS_RUN1
    {SPR_POSS,0,4,A_Chase,S_POSS_RUN3,0,0},	// S_POSS_RUN2
    {SPR_POSS,1,4,A_Chase,S_POSS_RUN4,0,0},	// S_POSS_RUN3
    {SPR_POSS,1,4,A_Chase,S_POSS_RUN5,0,0},	// S_POSS_RUN4
    {SPR_POSS,2,4,A_Chase,S_POSS_RUN6,0,0},	// S_POSS_RUN5
    {SPR_POSS,2,4,A_Chase,S_POSS_RUN7,0,0},	// S_POSS_RUN6
    {SPR_POSS,3,4,A_Chase,S_POSS_RUN8,0,0},	// S_POSS_RUN7
    {SPR_POSS,3,4,A_Chase,S_POSS_RUN1,0,0},	// S_POSS_RUN8
    {SPR_POSS,4,10,A_FaceTarget,S_POSS_ATK2,0,0},	// S_POSS_ATK1
    {SPR_POSS,5,8,A_PosAttack,S_POSS_ATK3,0,0},	// S_POSS_ATK2
    {SPR_POSS,4,8,A_NULL,S_POSS_RUN1,0,0},	// S_POSS_ATK3
    {SPR_POSS,6,3,A_NULL,S_POSS_PAIN2,0,0},	// S_POSS_PAIN
    {SPR_POSS,6,3,A_Pain,S_POSS_RUN1,0,0},	// S_POSS_PAIN2
    {SPR_POSS,7,5,A_NULL,S_POSS_DIE2,0,0},	// S_POSS_DIE1
    {SPR_POSS,8,5,A_Scream,S_POSS_DIE3,0,0},	// S_POSS_DIE2
    {SPR_POSS,9,5,A_Fall,S_POSS_DIE4,0,0},	// S_POSS_DIE3
    {SPR_POSS,10,5,A_NULL,S_POSS_DIE5,0,0},	// S_POSS_DIE4
    {SPR_POSS,11,-1,A_NULL,S_NULL,0,0},	// S_POSS_DIE5
    {SPR_POSS,12,5,A_NULL,S_POSS_XDIE2,0,0},	// S_POSS_XDIE1
    {SPR_POSS,13,5,A_XScream,S_POSS_XDIE3,0,0},	// S_POSS_XDIE2
    {SPR_POSS,14,5,A_Fall,S_POSS_XDIE4,0,0},	// S_POSS_XDIE3
    {SPR_POSS,15,5,A_NULL,S_POSS_XDIE5,0,0},	// S_POSS_XDIE4
    {SPR_POSS,16,5,A_NULL,S_POSS_XDIE6,0,0},	// S_POSS_XDIE5
    {SPR_POSS,17,5,A_NULL,S_POSS_XDIE7,0,0},	// S_POSS_XDIE6
    {SPR_POSS,18,5,A_NULL,S_POSS_XDIE8,0,0},	// S_POSS_XDIE7
    {SPR_POSS,19,5,A_NULL,S_POSS_XDIE9,0,0},	// S_POSS_XDIE8
    {SPR_POSS,20,-1,A_NULL,S_NULL,0,0},	// S_POSS_XDIE9
    {SPR_POSS,10,5,A_NULL,S_POSS_RAISE2,0,0},	// S_POSS_RAISE1
    {SPR_POSS,9,5,A_NULL,S_POSS_RAISE3,0,0},	// S_POSS_RAISE2
    {SPR_POSS,8,5,A_NULL,S_POSS_RAISE4,0,0},	// S_POSS_RAISE3
    {SPR_POSS,7,5,A_NULL,S_POSS_RUN1,0,0},	// S_POSS_RAISE4
    {SPR_SPOS,0,10,A_Look,S_SPOS_STND2,0,0},	// S_SPOS_STND
    {SPR_SPOS,1,10,A_Look,S_SPOS_STND,0,0},	// S_SPOS_STND2
    {SPR_SPOS,0,3,A_Chase,S_SPOS_RUN2,0,0},	// S_SPOS_RUN1
    {SPR_SPOS,0,3,A_Chase,S_SPOS_RUN3,0,0},	// S_SPOS_RUN2
    {SPR_SPOS,1,3,A_Chase,S_SPOS_RUN4,0,0},	// S_SPOS_RUN3
    {SPR_SPOS,1,3,A_Chase,S_SPOS_RUN5,0,0},	// S_SPOS_RUN4
    {SPR_SPOS,2,3,A_Chase,S_SPOS_RUN6,0,0},	// S_SPOS_RUN5
    {SPR_SPOS,2,3,A_Chase,S_SPOS_RUN7,0,0},	// S_SPOS_RUN6
    {SPR_SPOS,3,3,A_Chase,S_SPOS_RUN8,0,0},	// S_SPOS_RUN7
    {SPR_SPOS,3,3,A_Chase,S_SPOS_RUN1,0,0},	// S_SPOS_RUN8
    {SPR_SPOS,4,10,A_FaceTarget,S_SPOS_ATK2,0,0},	// S_SPOS_ATK1
    {SPR_SPOS,32773,10,A_SPosAttack,S_SPOS_ATK3,0,0},	// S_SPOS_ATK2
    {SPR_SPOS,4,10,A_NULL,S_SPOS_RUN1,0,0},	// S_SPOS_ATK3
    {SPR_SPOS,6,3,A_NULL,S_SPOS_PAIN2,0,0},	// S_SPOS_PAIN
    {SPR_SPOS,6,3,A_Pain,S_SPOS_RUN1,0,0},	// S_SPOS_PAIN2
    {SPR_SPOS,7,5,A_NULL,S_SPOS_DIE2,0,0},	// S_SPOS_DIE1
    {SPR_SPOS,8,5,A_Scream,S_SPOS_DIE3,0,0},	// S_SPOS_DIE2
    {SPR_SPOS,9,5,A_Fall,S_SPOS_DIE4,0,0},	// S_SPOS_DIE3
    {SPR_SPOS,10,5,A_NULL,S_SPOS_DIE5,0,0},	// S_SPOS_DIE4
    {SPR_SPOS,11,-1,A_NULL,S_NULL,0,0},	// S_SPOS_DIE5
    {SPR_SPOS,12,5,A_NULL,S_SPOS_XDIE2,0,0},	// S_SPOS_XDIE1
    {SPR_SPOS,13,5,A_XScream,S_SPOS_XDIE3,0,0},	// S_SPOS_XDIE2
    {SPR_SPOS,14,5,A_Fall,S_SPOS_XDIE4,0,0},	// S_SPOS_XDIE3
    {SPR_SPOS,15,5,A_NULL,S_SPOS_XDIE5,0,0},	// S_SPOS_XDIE4
    {SPR_SPOS,16,5,A_NULL,S_SPOS_XDIE6,0,0},	// S_SPOS_XDIE5
    {SPR_SPOS,17,5,A_NULL,S_SPOS_XDIE7,0,0},	// S_SPOS_XDIE6
    {SPR_SPOS,18,5,A_NULL,S_SPOS_XDIE8,0,0},	// S_SPOS_XDIE7
    {SPR_SPOS,19,5,A_NULL,S_SPOS_XDIE9,0,0},	// S_SPOS_XDIE8
    {SPR_SPOS,20,-1,A_NULL,S_NULL,0,0},	// S_SPOS_XDIE9
    {SPR_SPOS,11,5,A_NULL,S_SPOS_RAISE2,0,0},	// S_SPOS_RAISE1
    {SPR_SPOS,10,5,A_NULL,S_SPOS_RAISE3,0,0},	// S_SPOS_RAISE2
    {SPR_SPOS,9,5,A_NULL,S_SPOS_RAISE4,0,0},	// S_SPOS_RAISE3
    {SPR_SPOS,8,5,A_NULL,S_SPOS_RAISE5,0,0},	// S_SPOS_RAISE4
    {SPR_SPOS,7,5,A_NULL,S_SPOS_RUN1,0,0},	// S_SPOS_RAISE5
    {SPR_VILE,0,10,A_Look,S_VILE_STND2,0,0},	// S_VILE_STND
    {SPR_VILE,1,10,A_Look,S_VILE_STND,0,0},	// S_VILE_STND2
    {SPR_VILE,0,2,A_VileChase,S_VILE_RUN2,0,0},	// S_VILE_RUN1
    {SPR_VILE,0,2,A_VileChase,S_VILE_RUN3,0,0},	// S_VILE_RUN2
    {SPR_VILE,1,2,A_VileChase,S_VILE_RUN4,0,0},	// S_VILE_RUN3
    {SPR_VILE,1,2,A_VileChase,S_VILE_RUN5,0,0},	// S_VILE_RUN4
    {SPR_VILE,2,2,A_VileChase,S_VILE_RUN6,0,0},	// S_VILE_RUN5
    {SPR_VILE,2,2,A_VileChase,S_VILE_RUN7,0,0},	// S_VILE_RUN6
    {SPR_VILE,3,2,A_VileChase,S_VILE_RUN8,0,0},	// S_VILE_RUN7
    {SPR_VILE,3,2,A_VileChase,S_VILE_RUN9,0,0},	// S_VILE_RUN8
    {SPR_VILE,4,2,A_VileChase,S_VILE_RUN10,0,0},	// S_VILE_RUN9
    {SPR_VILE,4,2,A_VileChase,S_VILE_RUN11,0,0},	// S_VILE_RUN10
    {SPR_VILE,5,2,A_VileChase,S_VILE_RUN12,0,0},	// S_VILE_RUN11
    {SPR_VILE,5,2,A_VileChase,S_VILE_RUN1,0,0},	// S_VILE_RUN12
    {SPR_VILE,32774,0,A_VileStart,S_VILE_ATK2,0,0},	// S_VILE_ATK1
    {SPR_VILE,32774,10,A_FaceTarget,S_VILE_ATK3,0,0},	// S_VILE_ATK2
    {SPR_VILE,32775,8,A_VileTarget,S_VILE_ATK4,0,0},	// S_VILE_ATK3
    {SPR_VILE,32776,8,A_FaceTarget,S_VILE_ATK5,0,0},	// S_VILE_ATK4
    {SPR_VILE,32777,8,A_FaceTarget,S_VILE_ATK6,0,0},	// S_VILE_ATK5
    {SPR_VILE,32778,8,A_FaceTarget,S_VILE_ATK7,0,0},	// S_VILE_ATK6
    {SPR_VILE,32779,8,A_FaceTarget,S_VILE_ATK8,0,0},	// S_VILE_ATK7
    {SPR_VILE,32780,8,A_FaceTarget,S_VILE_ATK9,0,0},	// S_VILE_ATK8
    {SPR_VILE,32781,8,A_FaceTarget,S_VILE_ATK10,0,0},	// S_VILE_ATK9
    {SPR_VILE,32782,8,A_VileAttack,S_VILE_ATK11,0,0},	// S_VILE_ATK10
    {SPR_VILE,32783,20,A_NULL,S_VILE_RUN1,0,0},	// S_VILE_ATK11
    {SPR_VILE,32794,10,A_NULL,S_VILE_HEAL2,0,0},	// S_VILE_HEAL1
    {SPR_VILE,32795,10,A_NULL,S_VILE_HEAL3,0,0},	// S_VILE_HEAL2
    {SPR_VILE,32796,10,A_NULL,S_VILE_RUN1,0,0},	// S_VILE_HEAL3
    {SPR_VILE,16,5,A_NULL,S_VILE_PAIN2,0,0},	// S_VILE_PAIN
    {SPR_VILE,16,5,A_Pain,S_VILE_RUN1,0,0},	// S_VILE_PAIN2
    {SPR_VILE,16,7,A_NULL,S_VILE_DIE2,0,0},	// S_VILE_DIE1
    {SPR_VILE,17,7,A_Scream,S_VILE_DIE3,0,0},	// S_VILE_DIE2
    {SPR_VILE,18,7,A_Fall,S_VILE_DIE4,0,0},	// S_VILE_DIE3
    {SPR_VILE,19,7,A_NULL,S_VILE_DIE5,0,0},	// S_VILE_DIE4
    {SPR_VILE,20,7,A_NULL,S_VILE_DIE6,0,0},	// S_VILE_DIE5
    {SPR_VILE,21,7,A_NULL,S_VILE_DIE7,0,0},	// S_VILE_DIE6
    {SPR_VILE,22,7,A_NULL,S_VILE_DIE8,0,0},	// S_VILE_DIE7
    {SPR_VILE,23,5,A_NULL,S_VILE_DIE9,0,0},	// S_VILE_DIE8
    {SPR_VILE,24,5,A_NULL,S_VILE_DIE10,0,0},	// S_VILE_DIE9
    {SPR_VILE,25,-1,A_NULL,S_NULL,0,0},	// S_VILE_DIE10
    {SPR_FIRE,32768,2,A_StartFire,S_FIRE2,0,0},	// S_FIRE1
    {SPR_FIRE,32769,2,A_Fire,S_FIRE3,0,0},	// S_FIRE2
    {SPR_FIRE,32768,2,A_Fire,S_FIRE4,0,0},	// S_FIRE3
    {SPR_FIRE,32769,2,A_Fire,S_FIRE5,0,0},	// S_FIRE4
    {SPR_FIRE,32770,2,A_FireCrackle,S_FIRE6,0,0},	// S_FIRE5
    {SPR_FIRE,32769,2,A_Fire,S_FIRE7,0,0},	// S_FIRE6
    {SPR_FIRE,32770,2,A_Fire,S_FIRE8,0,0},	// S_FIRE7
    {SPR_FIRE,32769,2,A_Fire,S_FIRE9,0,0},	// S_FIRE8
    {SPR_FIRE,32770,2,A_Fire,S_FIRE10,0,0},	// S_FIRE9
    {SPR_FIRE,32771,2,A_Fire,S_FIRE11,0,0},	// S_FIRE10
    {SPR_FIRE,32770,2,A_Fire,S_FIRE12,0,0},	// S_FIRE11
    {SPR_FIRE,32771,2,A_Fire,S_FIRE13,0,0},	// S_FIRE12
    {SPR_FIRE,32770,2,A_Fire,S_FIRE14,0,0},	// S_FIRE13
    {SPR_FIRE,32771,2,A_Fire,S_FIRE15,0,0},	// S_FIRE14
    {SPR_FIRE,32772,2,A_Fire,S_FIRE16,0,0},	// S_FIRE15
    {SPR_FIRE,32771,2,A_Fire,S_FIRE17,0,0},	// S_FIRE16
    {SPR_FIRE,32772,2,A_Fire,S_FIRE18,0,0},	// S_FIRE17
    {SPR_FIRE,32771,2,A_Fire,S_FIRE19,0,0},	// S_FIRE18
    {SPR_FIRE,32772,2,A_FireCrackle,S_FIRE20,0,0},	// S_FIRE19
    {SPR_FIRE,32773,2,A_Fire,S_FIRE21,0,0},	// S_FIRE20
    {SPR_FIRE,32772,2,A_Fire,S_FIRE22,0,0},	// S_FIRE21
    {SPR_FIRE,32773,2,A_Fire,S_FIRE23,0,0},	// S_FIRE22
    {SPR_FIRE,32772,2,A_Fire,S_FIRE24,0,0},	// S_FIRE23
    {SPR_FIRE,32773,2,A_Fire,S_FIRE25,0,0},	// S_FIRE24
    {SPR_FIRE,32774,2,A_Fire,S_FIRE26,0,0},	// S_FIRE25
    {SPR_FIRE,32775,2,A_Fire,S_FIRE27,0,0},	// S_FIRE26
    {SPR_FIRE,32774,2,A_Fire,S_FIRE28,0,0},	// S_FIRE27
    {SPR_FIRE,32775,2,A_Fire,S_FIRE29,0,0},	// S_FIRE28
    {SPR_FIRE,32774,2,A_Fire,S_FIRE30,0,0},	// S_FIRE29
    {SPR_FIRE,32775,2,A_Fire,S_NULL,0,0},	// S_FIRE30
    {SPR_PUFF,1,4,A_NULL,S_SMOKE2,0,0},	// S_SMOKE1
    {SPR_PUFF,2,4,A_NULL,S_SMOKE3,0,0},	// S_SMOKE2
    {SPR_PUFF,1,4,A_NULL,S_SMOKE4,0,0},	// S_SMOKE3
    {SPR_PUFF,2,4,A_NULL,S_SMOKE5,0,0},	// S_SMOKE4
    {SPR_PUFF,3,4,A_NULL,S_NULL,0,0},	// S_SMOKE5
    {SPR_FATB,32768,2,A_Tracer,S_TRACER2,0,0},	// S_TRACER
    {SPR_FATB,32769,2,A_Tracer,S_TRACER,0,0},	// S_TRACER2
    {SPR_FBXP,32768,8,A_NULL,S_TRACEEXP2,0,0},	// S_TRACEEXP1
    {SPR_FBXP,32769,6,A_NULL,S_TRACEEXP3,0,0},	// S_TRACEEXP2
    {SPR_FBXP,32770,4,A_NULL,S_NULL,0,0},	// S_TRACEEXP3
    {SPR_SKEL,0,10,A_Look,S_SKEL_STND2,0,0},	// S_SKEL_STND
    {SPR_SKEL,1,10,A_Look,S_SKEL_STND,0,0},	// S_SKEL_STND2
    {SPR_SKEL,0,2,A_Chase,S_SKEL_RUN2,0,0},	// S_SKEL_RUN1
    {SPR_SKEL,0,2,A_Chase,S_SKEL_RUN3,0,0},	// S_SKEL_RUN2
    {SPR_SKEL,1,2,A_Chase,S_SKEL_RUN4,0,0},	// S_SKEL_RUN3
    {SPR_SKEL,1,2,A_Chase,S_SKEL_RUN5,0,0},	// S_SKEL_RUN4
    {SPR_SKEL,2,2,A_Chase,S_SKEL_RUN6,0,0},	// S_SKEL_RUN5
    {SPR_SKEL,2,2,A_Chase,S_SKEL_RUN7,0,0},	// S_SKEL_RUN6
    {SPR_SKEL,3,2,A_Chase,S_SKEL_RUN8,0,0},	// S_SKEL_RUN7
    {SPR_SKEL,3,2,A_Chase,S_SKEL_RUN9,0,0},	// S_SKEL_RUN8
    {SPR_SKEL,4,2,A_Chase,S_SKEL_RUN10,0,0},	// S_SKEL_RUN9
    {SPR_SKEL,4,2,A_Chase,S_SKEL_RUN11,0,0},	// S_SKEL_RUN10
    {SPR_SKEL,5,2,A_Chase,S_SKEL_RUN12,0,0},	// S_SKEL_RUN11
    {SPR_SKEL,5,2,A_Chase,S_SKEL_RUN1,0,0},	// S_SKEL_RUN12
    {SPR_SKEL,6,0,A_FaceTarget,S_SKEL_FIST2,0,0},	// S_SKEL_FIST1
    {SPR_SKEL,6,6,A_SkelWhoosh,S_SKEL_FIST3,0,0},	// S_SKEL_FIST2
    {SPR_SKEL,7,6,A_FaceTarget,S_SKEL_FIST4,0,0},	// S_SKEL_FIST3
    {SPR_SKEL,8,6,A_SkelFist,S_SKEL_RUN1,0,0},	// S_SKEL_FIST4
    {SPR_SKEL,32777,0,A_FaceTarget,S_SKEL_MISS2,0,0},	// S_SKEL_MISS1
    {SPR_SKEL,32777,10,A_FaceTarget,S_SKEL_MISS3,0,0},	// S_SKEL_MISS2
    {SPR_SKEL,10,10,A_SkelMissile,S_SKEL_MISS4,0,0},	// S_SKEL_MISS3
    {SPR_SKEL,10,10,A_FaceTarget,S_SKEL_RUN1,0,0},	// S_SKEL_MISS4
    {SPR_SKEL,11,5,A_NULL,S_SKEL_PAIN2,0,0},	// S_SKEL_PAIN
    {SPR_SKEL,11,5,A_Pain,S_SKEL_RUN1,0,0},	// S_SKEL_PAIN2
    {SPR_SKEL,11,7,A_NULL,S_SKEL_DIE2,0,0},	// S_SKEL_DIE1
    {SPR_SKEL,12,7,A_NULL,S_SKEL_DIE3,0,0},	// S_SKEL_DIE2
    {SPR_SKEL,13,7,A_Scream,S_SKEL_DIE4,0,0},	// S_SKEL_DIE3
    {SPR_SKEL,14,7,A_Fall,S_SKEL_DIE5,0,0},	// S_SKEL_DIE4
    {SPR_SKEL,15,7,A_NULL,S_SKEL_DIE6,0,0},	// S_SKEL_DIE5
    {SPR_SKEL,16,-1,A_NULL,S_NULL,0,0},	// S_SKEL_DIE6
    {SPR_SKEL,16,5,A_NULL,S_SKEL_RAISE2,0,0},	// S_SKEL_RAISE1
    {SPR_SKEL,15,5,A_NULL,S_SKEL_RAISE3,0,0},	// S_SKEL_RAISE2
    {SPR_SKEL,14,5,A_NULL,S_SKEL_RAISE4,0,0},	// S_SKEL_RAISE3
    {SPR_SKEL,13,5,A_NULL,S_SKEL_RAISE5,0,0},	// S_SKEL_RAISE4
    {SPR_SKEL,12,5,A_NULL,S_SKEL_RAISE6,0,0},	// S_SKEL_RAISE5
    {SPR_SKEL,11,5,A_NULL,S_SKEL_RUN1,0,0},	// S_SKEL_RAISE6
    {SPR_MANF,32768,4,A_NULL,S_FATSHOT2,0,0},	// S_FATSHOT1
    {SPR_MANF,32769,4,A_NULL,S_FATSHOT1,0,0},	// S_FATSHOT2
    {SPR_MISL,32769,8,A_NULL,S_FATSHOTX2,0,0},	// S_FATSHOTX1
    {SPR_MISL,32770,6,A_NULL,S_FATSHOTX3,0,0},	// S_FATSHOTX2
    {SPR_MISL,32771,4,A_NULL,S_NULL,0,0},	// S_FATSHOTX3
    {SPR_FATT,0,15,A_Look,S_FATT_STND2,0,0},	// S_FATT_STND
    {SPR_FATT,1,15,A_Look,S_FATT_STND,0,0},	// S_FATT_STND2
    {SPR_FATT,0,4,A_Chase,S_FATT_RUN2,0,0},	// S_FATT_RUN1
    {SPR_FATT,0,4,A_Chase,S_FATT_RUN3,0,0},	// S_FATT_RUN2
    {SPR_FATT,1,4,A_Chase,S_FATT_RUN4,0,0},	// S_FATT_RUN3
    {SPR_FATT,1,4,A_Chase,S_FATT_RUN5,0,0},	// S_FATT_RUN4
    {SPR_FATT,2,4,A_Chase,S_FATT_RUN6,0,0},	// S_FATT_RUN5
    {SPR_FATT,2,4,A_Chase,S_FATT_RUN7,0,0},	// S_FATT_RUN6
    {SPR_FATT,3,4,A_Chase,S_FATT_RUN8,0,0},	// S_FATT_RUN7
    {SPR_FATT,3,4,A_Chase,S_FATT_RUN9,0,0},	// S_FATT_RUN8
    {SPR_FATT,4,4,A_Chase,S_FATT_RUN10,0,0},	// S_FATT_RUN9
    {SPR_FATT,4,4,A_Chase,S_FATT_RUN11,0,0},	// S_FATT_RUN10
    {SPR_FATT,5,4,A_Chase,S_FATT_RUN12,0,0},	// S_FATT_RUN11
    {SPR_FATT,5,4,A_Chase,S_FATT_RUN1,0,0},	// S_FATT_RUN12
    {SPR_FATT,6,20,A_FatRaise,S_FATT_ATK2,0,0},	// S_FATT_ATK1
    {SPR_FATT,32775,10,A_FatAttack1,S_FATT_ATK3,0,0},	// S_FATT_ATK2
    {SPR_FATT,8,5,A_FaceTarget,S_FATT_ATK4,0,0},	// S_FATT_ATK3
    {SPR_FATT,6,5,A_FaceTarget,S_FATT_ATK5,0,0},	// S_FATT_ATK4
    {SPR_FATT,32775,10,A_FatAttack2,S_FATT_ATK6,0,0},	// S_FATT_ATK5
    {SPR_FATT,8,5,A_FaceTarget,S_FATT_ATK7,0,0},	// S_FATT_ATK6
    {SPR_FATT,6,5,A_FaceTarget,S_FATT_ATK8,0,0},	// S_FATT_ATK7
    {SPR_FATT,32775,10,A_FatAttack3,S_FATT_ATK9,0,0},	// S_FATT_ATK8
    {SPR_FATT,8,5,A_FaceTarget,S_FATT_ATK10,0,0},	// S_FATT_ATK9
    {SPR_FATT,6,5,A_FaceTarget,S_FATT_RUN1,0,0},	// S_FATT_ATK10
    {SPR_FATT,9,3,A_NULL,S_FATT_PAIN2,0,0},	// S_FATT_PAIN
    {SPR_FATT,9,3,A_Pain,S_FATT_RUN1,0,0},	// S_FATT_PAIN2
    {SPR_FATT,10,6,A_NULL,S_FATT_DIE2,0,0},	// S_FATT_DIE1
    {SPR_FATT,11,6,A_Scream,S_FATT_DIE3,0,0},	// S_FATT_DIE2
    {SPR_FATT,12,6,A_Fall,S_FATT_DIE4,0,0},	// S_FATT_DIE3
    {SPR_FATT,13,6,A_NULL,S_FATT_DIE5,0,0},	// S_FATT_DIE4
    {SPR_FATT,14,6,A_NULL,S_FATT_DIE6,0,0},	// S_FATT_DIE5
    {SPR_FATT,15,6,A_NULL,S_FATT_DIE7,0,0},	// S_FATT_DIE6
    {SPR_FATT,16,6,A_NULL,S_FATT_DIE8,0,0},	// S_FATT_DIE7
    {SPR_FATT,17,6,A_NULL,S_FATT_DIE9,0,0},	// S_FATT_DIE8
    {SPR_FATT,18,6,A_NULL,S_FATT_DIE10,0,0},	// S_FATT_DIE9
    {SPR_FATT,19,-1,A_BossDeath,S_NULL,0,0},	// S_FATT_DIE10
    {SPR_FATT,17,5,A_NULL,S_FATT_RAISE2,0,0},	// S_FATT_RAISE1
    {SPR_FATT,16,5,A_NULL,S_FATT_RAISE3,0,0},	// S_FATT_RAISE2
    {SPR_FATT,15,5,A_NULL,S_FATT_RAISE4,0,0},	// S_FATT_RAISE3
    {SPR_FATT,14,5,A_NULL,S_FATT_RAISE5,0,0},	// S_FATT_RAISE4
    {SPR_FATT,13,5,A_NULL,S_FATT_RAISE6,0,0},	// S_FATT_RAISE5
    {SPR_FATT,12,5,A_NULL,S_FATT_RAISE7,0,0},	// S_FATT_RAISE6
    {SPR_FATT,11,5,A_NULL,S_FATT_RAISE8,0,0},	// S_FATT_RAISE7
    {SPR_FATT,10,5,A_NULL,S_FATT_RUN1,0,0},	// S_FATT_RAISE8
    {SPR_CPOS,0,10,A_Look,S_CPOS_STND2,0,0},	// S_CPOS_STND
    {SPR_CPOS,1,10,A_Look,S_CPOS_STND,0,0},	// S_CPOS_STND2
    {SPR_CPOS,0,3,A_Chase,S_CPOS_RUN2,0,0},	// S_CPOS_RUN1
    {SPR_CPOS,0,3,A_Chase,S_CPOS_RUN3,0,0},	// S_CPOS_RUN2
    {SPR_CPOS,1,3,A_Chase,S_CPOS_RUN4,0,0},	// S_CPOS_RUN3
    {SPR_CPOS,1,3,A_Chase,S_CPOS_RUN5,0,0},	// S_CPOS_RUN4
    {SPR_CPOS,2,3,A_Chase,S_CPOS_RUN6,0,0},	// S_CPOS_RUN5
    {SPR_CPOS,2,3,A_Chase,S_CPOS_RUN7,0,0},	// S_CPOS_RUN6
    {SPR_CPOS,3,3,A_Chase,S_CPOS_RUN8,0,0},	// S_CPOS_RUN7
    {SPR_CPOS,3,3,A_Chase,S_CPOS_RUN1,0,0},	// S_CPOS_RUN8
    {SPR_CPOS,4,10,A_FaceTarget,S_CPOS_ATK2,0,0},	// S_CPOS_ATK1
    {SPR_CPOS,32773,4,A_CPosAttack,S_CPOS_ATK3,0,0},	// S_CPOS_ATK2
    {SPR_CPOS,32772,4,A_CPosAttack,S_CPOS_ATK4,0,0},	// S_CPOS_ATK3
    {SPR_CPOS,5,1,A_CPosRefire,S_CPOS_ATK2,0,0},	// S_CPOS_ATK4
    {SPR_CPOS,6,3,A_NULL,S_CPOS_PAIN2,0,0},	// S_CPOS_PAIN
    {SPR_CPOS,6,3,A_Pain,S_CPOS_RUN1,0,0},	// S_CPOS_PAIN2
    {SPR_CPOS,7,5,A_NULL,S_CPOS_DIE2,0,0},	// S_CPOS_DIE1
    {SPR_CPOS,8,5,A_Scream,S_CPOS_DIE3,0,0},	// S_CPOS_DIE2
    {SPR_CPOS,9,5,A_Fall,S_CPOS_DIE4,0,0},	// S_CPOS_DIE3
    {SPR_CPOS,10,5,A_NULL,S_CPOS_DIE5,0,0},	// S_CPOS_DIE4
    {SPR_CPOS,11,5,A_NULL,S_CPOS_DIE6,0,0},	// S_CPOS_DIE5
    {SPR_CPOS,12,5,A_NULL,S_CPOS_DIE7,0,0},	// S_CPOS_DIE6
    {SPR_CPOS,13,-1,A_NULL,S_NULL,0,0},	// S_CPOS_DIE7
    {SPR_CPOS,14,5,A_NULL,S_CPOS_XDIE2,0,0},	// S_CPOS_XDIE1
    {SPR_CPOS,15,5,A_XScream,S_CPOS_XDIE3,0,0},	// S_CPOS_XDIE2
    {SPR_CPOS,16,5,A_Fall,S_CPOS_XDIE4,0,0},	// S_CPOS_XDIE3
    {SPR_CPOS,17,5,A_NULL,S_CPOS_XDIE5,0,0},	// S_CPOS_XDIE4
    {SPR_CPOS,18,5,A_NULL,S_CPOS_XDIE6,0,0},	// S_CPOS_XDIE5
    {SPR_CPOS,19,-1,A_NULL,S_NULL,0,0},	// S_CPOS_XDIE6
    {SPR_CPOS,13,5,A_NULL,S_CPOS_RAISE2,0,0},	// S_CPOS_RAISE1
    {SPR_CPOS,12,5,A_NULL,S_CPOS_RAISE3,0,0},	// S_CPOS_RAISE2
    {SPR_CPOS,11,5,A_NULL,S_CPOS_RAISE4,0,0},	// S_CPOS_RAISE3
    {SPR_CPOS,10,5,A_NULL,S_CPOS_RAISE5,0,0},	// S_CPOS_RAISE4
    {SPR_CPOS,9,5,A_NULL,S_CPOS_RAISE6,0,0},	// S_CPOS_RAISE5
    {SPR_CPOS,8,5,A_NULL,S_CPOS_RAISE7,0,0},	// S_CPOS_RAISE6
    {SPR_CPOS,7,5,A_NULL,S_CPOS_RUN1,0,0},	// S_CPOS_RAISE7
    {SPR_TROO,0,10,A_Look,S_TROO_STND2,0,0},	// S_TROO_STND
    {SPR_TROO,1,10,A_Look,S_TROO_STND,0,0},	// S_TROO_STND2
    {SPR_TROO,0,3,A_Chase,S_TROO_RUN2,0,0},	// S_TROO_RUN1
    {SPR_TROO,0,3,A_Chase,S_TROO_RUN3,0,0},	// S_TROO_RUN2
    {SPR_TROO,1,3,A_Chase,S_TROO_RUN4,0,0},	// S_TROO_RUN3
    {SPR_TROO,1,3,A_Chase,S_TROO_RUN5,0,0},	// S_TROO_RUN4
    {SPR_TROO,2,3,A_Chase,S_TROO_RUN6,0,0},	// S_TROO_RUN5
    {SPR_TROO,2,3,A_Chase,S_TROO_RUN7,0,0},	// S_TROO_RUN6
    {SPR_TROO,3,3,A_Chase,S_TROO_RUN8,0,0},	// S_TROO_RUN7
    {SPR_TROO,3,3,A_Chase,S_TROO_RUN1,0,0},	// S_TROO_RUN8
    {SPR_TROO,4,8,A_FaceTarget,S_TROO_ATK2,0,0},	// S_TROO_ATK1
    {SPR_TROO,5,8,A_FaceTarget,S_TROO_ATK3,0,0},	// S_TROO_ATK2
    {SPR_TROO,6,6,A_TroopAttack,S_TROO_RUN1,0,0},	// S_TROO_ATK3
    {SPR_TROO,7,2,A_NULL,S_TROO_PAIN2,0,0},	// S_TROO_PAIN
    {SPR_TROO,7,2,A_Pain,S_TROO_RUN1,0,0},	// S_TROO_PAIN2
    {SPR_TROO,8,8,A_NULL,S_TROO_DIE2,0,0},	// S_TROO_DIE1
    {SPR_TROO,9,8,A_Scream,S_TROO_DIE3,0,0},	// S_TROO_DIE2
    {SPR_TROO,10,6,A_NULL,S_TROO_DIE4,0,0},	// S_TROO_DIE3
    {SPR_TROO,11,6,A_Fall,S_TROO_DIE5,0,0},	// S_TROO_DIE4
    {SPR_TROO,12,-1,A_NULL,S_NULL,0,0},	// S_TROO_DIE5
    {SPR_TROO,13,5,A_NULL,S_TROO_XDIE2,0,0},	// S_TROO_XDIE1
    {SPR_TROO,14,5,A_XScream,S_TROO_XDIE3,0,0},	// S_TROO_XDIE2
    {SPR_TROO,15,5,A_NULL,S_TROO_XDIE4,0,0},	// S_TROO_XDIE3
    {SPR_TROO,16,5,A_Fall,S_TROO_XDIE5,0,0},	// S_TROO_XDIE4
    {SPR_TROO,17,5,A_NULL,S_TROO_XDIE6,0,0},	// S_TROO_XDIE5
    {SPR_TROO,18,5,A_NULL,S_TROO_XDIE7,0,0},	// S_TROO_XDIE6
    {SPR_TROO,19,5,A_NULL,S_TROO_XDIE8,0,0},	// S_TROO_XDIE7
    {SPR_TROO,20,-1,A_NULL,S_NULL,0,0},	// S_TROO_XDIE8
    {SPR_TROO,12,8,A_NULL,S_TROO_RAISE2,0,0},	// S_TROO_RAISE1
    {SPR_TROO,11,8,A_NULL,S_TROO_RAISE3,0,0},	// S_TROO_RAISE2
    {SPR_TROO,10,6,A_NULL,S_TROO_RAISE4,0,0},	// S_TROO_RAISE3
    {SPR_TROO,9,6,A_NULL,S_TROO_RAISE5,0,0},	// S_TROO_RAISE4
    {SPR_TROO,8,6,A_NULL,S_TROO_RUN1,0,0},	// S_TROO_RAISE5
    {SPR_SARG,0,10,A_Look,S_SARG_STND2,0,0},	// S_SARG_STND
    {SPR_SARG,1,10,A_Look,S_SARG_STND,0,0},	// S_SARG_STND2
    {SPR_SARG,0,2,A_Chase,S_SARG_RUN2,0,0},	// S_SARG_RUN1
    {SPR_SARG,0,2,A_Chase,S_SARG_RUN3,0,0},	// S_SARG_RUN2
    {SPR_SARG,1,2,A_Chase,S_SARG_RUN4,0,0},	// S_SARG_RUN3
    {SPR_SARG,1,2,A_Chase,S_SARG_RUN5,0,0},	// S_SARG_RUN4
    {SPR_SARG,2,2,A_Chase,S_SARG_RUN6,0,0},	// S_SARG_RUN5
    {SPR_SARG,2,2,A_Chase,S_SARG_RUN7,0,0},	// S_SARG_RUN6
    {SPR_SARG,3,2,A_Chase,S_SARG_RUN8,0,0},	// S_SARG_RUN7
    {SPR_SARG,3,2,A_Chase,S_SARG_RUN1,0,0},	// S_SARG_RUN8
    {SPR_SARG,4,8,A_FaceTarget,S_SARG_ATK2,0,0},	// S_SARG_ATK1
    {SPR_SARG,5,8,A_FaceTarget,S_SARG_ATK3,0,0},	// S_SARG_ATK2
    {SPR_SARG,6,8,A_SargAttack,S_SARG_RUN1,0,0},	// S_SARG_ATK3
    {SPR_SARG,7,2,A_NULL,S_SARG_PAIN2,0,0},	// S_SARG_PAIN
    {SPR_SARG,7,2,A_Pain,S_SARG_RUN1,0,0},	// S_SARG_PAIN2
    {SPR_SARG,8,8,A_NULL,S_SARG_DIE2,0,0},	// S_SARG_DIE1
    {SPR_SARG,9,8,A_Scream,S_SARG_DIE3,0,0},	// S_SARG_DIE2
    {SPR_SARG,10,4,A_NULL,S_SARG_DIE4,0,0},	// S_SARG_DIE3
    {SPR_SARG,11,4,A_Fall,S_SARG_DIE5,0,0},	// S_SARG_DIE4
    {SPR_SARG,12,4,A_NULL,S_SARG_DIE6,0,0},	// S_SARG_DIE5
    {SPR_SARG,13,-1,A_NULL,S_NULL,0,0},	// S_SARG_DIE6
    {SPR_SARG,13,5,A_NULL,S_SARG_RAISE2,0,0},	// S_SARG_RAISE1
    {SPR_SARG,12,5,A_NULL,S_SARG_RAISE3,0,0},	// S_SARG_RAISE2
    {SPR_SARG,11,5,A_NULL,S_SARG_RAISE4,0,0},	// S_SARG_RAISE3
    {SPR_SARG,10,5,A_NULL,S_SARG_RAISE5,0,0},	// S_SARG_RAISE4
    {SPR_SARG,9,5,A_NULL,S_SARG_RAISE6,0,0},	// S_SARG_RAISE5
    {SPR_SARG,8,5,A_NULL,S_SARG_RUN1,0,0},	// S_SARG_RAISE6
    {SPR_HEAD,0,10,A_Look,S_HEAD_STND,0,0},	// S_HEAD_STND
    {SPR_HEAD,0,3,A_Chase,S_HEAD_RUN1,0,0},	// S_HEAD_RUN1
    {SPR_HEAD,1,5,A_FaceTarget,S_HEAD_ATK2,0,0},	// S_HEAD_ATK1
    {SPR_HEAD,2,5,A_FaceTarget,S_HEAD_ATK3,0,0},	// S_HEAD_ATK2
    {SPR_HEAD,32771,5,A_HeadAttack,S_HEAD_RUN1,0,0},	// S_HEAD_ATK3
    {SPR_HEAD,4,3,A_NULL,S_HEAD_PAIN2,0,0},	// S_HEAD_PAIN
    {SPR_HEAD,4,3,A_Pain,S_HEAD_PAIN3,0,0},	// S_HEAD_PAIN2
    {SPR_HEAD,5,6,A_NULL,S_HEAD_RUN1,0,0},	// S_HEAD_PAIN3
    {SPR_HEAD,6,8,A_NULL,S_HEAD_DIE2,0,0},	// S_HEAD_DIE1
    {SPR_HEAD,7,8,A_Scream,S_HEAD_DIE3,0,0},	// S_HEAD_DIE2
    {SPR_HEAD,8,8,A_NULL,S_HEAD_DIE4,0,0},	// S_HEAD_DIE3
    {SPR_HEAD,9,8,A_NULL,S_HEAD_DIE5,0,0},	// S_HEAD_DIE4
    {SPR_HEAD,10,8,A_Fall,S_HEAD_DIE6,0,0},	// S_HEAD_DIE5
    {SPR_HEAD,11,-1,A_NULL,S_NULL,0,0},	// S_HEAD_DIE6
    {SPR_HEAD,11,8,A_NULL,S_HEAD_RAISE2,0,0},	// S_HEAD_RAISE1
    {SPR_HEAD,10,8,A_NULL,S_HEAD_RAISE3,0,0},	// S_HEAD_RAISE2
    {SPR_HEAD,9,8,A_NULL,S_HEAD_RAISE4,0,0},	// S_HEAD_RAISE3
    {SPR_HEAD,8,8,A_NULL,S_HEAD_RAISE5,0,0},	// S_HEAD_RAISE4
    {SPR_HEAD,7,8,A_NULL,S_HEAD_RAISE6,0,0},	// S_HEAD_RAISE5
    {SPR_HEAD,6,8,A_NULL,S_HEAD_RUN1,0,0},	// S_HEAD_RAISE6
    {SPR_BAL7,32768,4,A_NULL,S_BRBALL2,0,0},	// S_BRBALL1
    {SPR_BAL7,32769,4,A_NULL,S_BRBALL1,0,0},	// S_BRBALL2
    {SPR_BAL7,32770,6,A_NULL,S_BRBALLX2,0,0},	// S_BRBALLX1
    {SPR_BAL7,32771,6,A_NULL,S_BRBALLX3,0,0},	// S_BRBALLX2
    {SPR_BAL7,32772,6,A_NULL,S_NULL,0,0},	// S_BRBALLX3
    {SPR_BOSS,0,10,A_Look,S_BOSS_STND2,0,0},	// S_BOSS_STND
    {SPR_BOSS,1,10,A_Look,S_BOSS_STND,0,0},	// S_BOSS_STND2
    {SPR_BOSS,0,3,A_Chase,S_BOSS_RUN2,0,0},	// S_BOSS_RUN1
    {SPR_BOSS,0,3,A_Chase,S_BOSS_RUN3,0,0},	// S_BOSS_RUN2
    {SPR_BOSS,1,3,A_Chase,S_BOSS_RUN4,0,0},	// S_BOSS_RUN3
    {SPR_BOSS,1,3,A_Chase,S_BOSS_RUN5,0,0},	// S_BOSS_RUN4
    {SPR_BOSS,2,3,A_Chase,S_BOSS_RUN6,0,0},	// S_BOSS_RUN5
    {SPR_BOSS,2,3,A_Chase,S_BOSS_RUN7,0,0},	// S_BOSS_RUN6
    {SPR_BOSS,3,3,A_Chase,S_BOSS_RUN8,0,0},	// S_BOSS_RUN7
    {SPR_BOSS,3,3,A_Chase,S_BOSS_RUN1,0,0},	// S_BOSS_RUN8
    {SPR_BOSS,4,8,A_FaceTarget,S_BOSS_ATK2,0,0},	// S_BOSS_ATK1
    {SPR_BOSS,5,8,A_FaceTarget,S_BOSS_ATK3,0,0},	// S_BOSS_ATK2
    {SPR_BOSS,6,8,A_BruisAttack,S_BOSS_RUN1,0,0},	// S_BOSS_ATK3
    {SPR_BOSS,7,2,A_NULL,S_BOSS_PAIN2,0,0},	// S_BOSS_PAIN
    {SPR_BOSS,7,2,A_Pain,S_BOSS_RUN1,0,0},	// S_BOSS_PAIN2
    {SPR_BOSS,8,8,A_NULL,S_BOSS_DIE2,0,0},	// S_BOSS_DIE1
    {SPR_BOSS,9,8,A_Scream,S_BOSS_DIE3,0,0},	// S_BOSS_DIE2
    {SPR_BOSS,10,8,A_NULL,S_BOSS_DIE4,0,0},	// S_BOSS_DIE3
    {SPR_BOSS,11,8,A_Fall,S_BOSS_DIE5,0,0},	// S_BOSS_DIE4
    {SPR_BOSS,12,8,A_NULL,S_BOSS_DIE6,0,0},	// S_BOSS_DIE5
    {SPR_BOSS,13,8,A_NULL,S_BOSS_DIE7,0,0},	// S_BOSS_DIE6
    {SPR_BOSS,14,-1,A_BossDeath,S_NULL,0,0},	// S_BOSS_DIE7
    {SPR_BOSS,14,8,A_NULL,S_BOSS_RAISE2,0,0},	// S_BOSS_RAISE1
    {SPR_BOSS,13,8,A_NULL,S_BOSS_RAISE3,0,0},	// S_BOSS_RAISE2
    {SPR_BOSS,12,8,A_NULL,S_BOSS_RAISE4,0,0},	// S_BOSS_RAISE3
    {SPR_BOSS,11,8,A_NULL,S_BOSS_RAISE5,0,0},	// S_BOSS_RAISE4
    {SPR_BOSS,10,8,A_NULL,S_BOSS_RAISE6,0,0},	// S_BOSS_RAISE5
    {SPR_BOSS,9,8,A_NULL,S_BOSS_RAISE7,0,0},	// S_BOSS_RAISE6
    {SPR_BOSS,8,8,A_NULL,S_BOSS_RUN1,0,0},	// S_BOSS_RAISE7
    {SPR_BOS2,0,10,A_Look,S_BOS2_STND2,0,0},	// S_BOS2_STND
    {SPR_BOS2,1,10,A_Look,S_BOS2_STND,0,0},	// S_BOS2_STND2
    {SPR_BOS2,0,3,A_Chase,S_BOS2_RUN2,0,0},	// S_BOS2_RUN1
    {SPR_BOS2,0,3,A_Chase,S_BOS2_RUN3,0,0},	// S_BOS2_RUN2
    {SPR_BOS2,1,3,A_Chase,S_BOS2_RUN4,0,0},	// S_BOS2_RUN3
    {SPR_BOS2,1,3,A_Chase,S_BOS2_RUN5,0,0},	// S_BOS2_RUN4
    {SPR_BOS2,2,3,A_Chase,S_BOS2_RUN6,0,0},	// S_BOS2_RUN5
    {SPR_BOS2,2,3,A_Chase,S_BOS2_RUN7,0,0},	// S_BOS2_RUN6
    {SPR_BOS2,3,3,A_Chase,S_BOS2_RUN8,0,0},	// S_BOS2_RUN7
    {SPR_BOS2,3,3,A_Chase,S_BOS2_RUN1,0,0},	// S_BOS2_RUN8
    {SPR_BOS2,4,8,A_FaceTarget,S_BOS2_ATK2,0,0},	// S_BOS2_ATK1
    {SPR_BOS2,5,8,A_FaceTarget,S_BOS2_ATK3,0,0},	// S_BOS2_ATK2
    {SPR_BOS2,6,8,A_BruisAttack,S_BOS2_RUN1,0,0},	// S_BOS2_ATK3
    {SPR_BOS2,7,2,A_NULL,S_BOS2_PAIN2,0,0},	// S_BOS2_PAIN
    {SPR_BOS2,7,2,A_Pain,S_BOS2_RUN1,0,0},	// S_BOS2_PAIN2
    {SPR_BOS2,8,8,A_NULL,S_BOS2_DIE2,0,0},	// S_BOS2_DIE1
    {SPR_BOS2,9,8,A_Scream,S_BOS2_DIE3,0,0},	// S_BOS2_DIE2
    {SPR_BOS2,10,8,A_NULL,S_BOS2_DIE4,0,0},	// S_BOS2_DIE3
    {SPR_BOS2,11,8,A_Fall,S_BOS2_DIE5,0,0},	// S_BOS2_DIE4
    {SPR_BOS2,12,8,A_NULL,S_BOS2_DIE6,0,0},	// S_BOS2_DIE5
    {SPR_BOS2,13,8,A_NULL,S_BOS2_DIE7,0,0},	// S_BOS2_DIE6
    {SPR_BOS2,14,-1,A_NULL,S_NULL,0,0},	// S_BOS2_DIE7
    {SPR_BOS2,14,8,A_NULL,S_BOS2_RAISE2,0,0},	// S_BOS2_RAISE1
    {SPR_BOS2,13,8,A_NULL,S_BOS2_RAISE3,0,0},	// S_BOS2_RAISE2
    {SPR_BOS2,12,8,A_NULL,S_BOS2_RAISE4,0,0},	// S_BOS2_RAISE3
    {SPR_BOS2,11,8,A_NULL,S_BOS2_RAISE5,0,0},	// S_BOS2_RAISE4
    {SPR_BOS2,10,8,A_NULL,S_BOS2_RAISE6,0,0},	// S_BOS2_RAISE5
    {SPR_BOS2,9,8,A_NULL,S_BOS2_RAISE7,0,0},	// S_BOS2_RAISE6
    {SPR_BOS2,8,8,A_NULL,S_BOS2_RUN1,0,0},	// S_BOS2_RAISE7
    {SPR_SKUL,32768,10,A_Look,S_SKULL_STND2,0,0},	// S_SKULL_STND
    {SPR_SKUL,32769,10,A_Look,S_SKULL_STND,0,0},	// S_SKULL_STND2
    {SPR_SKUL,32768,6,A_Chase,S_SKULL_RUN2,0,0},	// S_SKULL_RUN1
    {SPR_SKUL,32769,6,A_Chase,S_SKULL_RUN1,0,0},	// S_SKULL_RUN2
    {SPR_SKUL,32770,10,A_FaceTarget,S_SKULL_ATK2,0,0},	// S_SKULL_ATK1
    {SPR_SKUL,32771,4,A_SkullAttack,S_SKULL_ATK3,0,0},	// S_SKULL_ATK2
    {SPR_SKUL,32770,4,A_NULL,S_SKULL_ATK4,0,0},	// S_SKULL_ATK3
    {SPR_SKUL,32771,4,A_NULL,S_SKULL_ATK3,0,0},	// S_SKULL_ATK4
    {SPR_SKUL,32772,3,A_NULL,S_SKULL_PAIN2,0,0},	// S_SKULL_PAIN
    {SPR_SKUL,32772,3,A_Pain,S_SKULL_RUN1,0,0},	// S_SKULL_PAIN2
    {SPR_SKUL,32773,6,A_NULL,S_SKULL_DIE2,0,0},	// S_SKULL_DIE1
    {SPR_SKUL,32774,6,A_Scream,S_SKULL_DIE3,0,0},	// S_SKULL_DIE2
    {SPR_SKUL,32775,6,A_NULL,S_SKULL_DIE4,0,0},	// S_SKULL_DIE3
    {SPR_SKUL,32776,6,A_Fall,S_SKULL_DIE5,0,0},	// S_SKULL_DIE4
    {SPR_SKUL,9,6,A_NULL,S_SKULL_DIE6,0,0},	// S_SKULL_DIE5
    {SPR_SKUL,10,6,A_NULL,S_NULL,0,0},	// S_SKULL_DIE6
    {SPR_SPID,0,10,A_Look,S_SPID_STND2,0,0},	// S_SPID_STND
    {SPR_SPID,1,10,A_Look,S_SPID_STND,0,0},	// S_SPID_STND2
    {SPR_SPID,0,3,A_Metal,S_SPID_RUN2,0,0},	// S_SPID_RUN1
    {SPR_SPID,0,3,A_Chase,S_SPID_RUN3,0,0},	// S_SPID_RUN2
    {SPR_SPID,1,3,A_Chase,S_SPID_RUN4,0,0},	// S_SPID_RUN3
    {SPR_SPID,1,3,A_Chase,S_SPID_RUN5,0,0},	// S_SPID_RUN4
    {SPR_SPID,2,3,A_Metal,S_SPID_RUN6,0,0},	// S_SPID_RUN5
    {SPR_SPID,2,3,A_Chase,S_SPID_RUN7,0,0},	// S_SPID_RUN6
    {SPR_SPID,3,3,A_Chase,S_SPID_RUN8,0,0},	// S_SPID_RUN7
    {SPR_SPID,3,3,A_Chase,S_SPID_RUN9,0,0},	// S_SPID_RUN8
    {SPR_SPID,4,3,A_Metal,S_SPID_RUN10,0,0},	// S_SPID_RUN9
    {SPR_SPID,4,3,A_Chase,S_SPID_RUN11,0,0},	// S_SPID_RUN10
    {SPR_SPID,5,3,A_Chase,S_SPID_RUN12,0,0},	// S_SPID_RUN11
    {SPR_SPID,5,3,A_Chase,S_SPID_RUN1,0,0},	// S_SPID_RUN12
    {SPR_SPID,32768,20,A_FaceTarget,S_SPID_ATK2,0,0},	// S_SPID_ATK1
    {SPR_SPID,32774,4,A_SPosAttack,S_SPID_ATK3,0,0},	// S_SPID_ATK2
    {SPR_SPID,32775,4,A_SPosAttack,S_SPID_ATK4,0,0},	// S_SPID_ATK3
    {SPR_SPID,32775,1,A_SpidRefire,S_SPID_ATK2,0,0},	// S_SPID_ATK4
    {SPR_SPID,8,3,A_NULL,S_SPID_PAIN2,0,0},	// S_SPID_PAIN
    {SPR_SPID,8,3,A_Pain,S_SPID_RUN1,0,0},	// S_SPID_PAIN2
    {SPR_SPID,9,20,A_Scream,S_SPID_DIE2,0,0},	// S_SPID_DIE1
    {SPR_SPID,10,10,A_Fall,S_SPID_DIE3,0,0},	// S_SPID_DIE2
    {SPR_SPID,11,10,A_NULL,S_SPID_DIE4,0,0},	// S_SPID_DIE3
    {SPR_SPID,12,10,A_NULL,S_SPID_DIE5,0,0},	// S_SPID_DIE4
    {SPR_SPID,13,10,A_NULL,S_SPID_DIE6,0,0},	// S_SPID_DIE5
    {SPR_SPID,14,10,A_NULL,S_SPID_DIE7,0,0},	// S_SPID_DIE6
    {SPR_SPID,15,10,A_NULL,S_SPID_DIE8,0,0},	// S_SPID_DIE7
    {SPR_SPID,16,10,A_NULL,S_SPID_DIE9,0,0},	// S_SPID_DIE8
    {SPR_SPID,17,10,A_NULL,S_SPID_DIE10,0,0},	// S_SPID_DIE9
    {SPR_SPID,18,30,A_NULL,S_SPID_DIE11,0,0},	// S_SPID_DIE10
    {SPR_SPID,18,-1,A_BossDeath,S_NULL,0,0},	// S_SPID_DIE11
    {SPR_BSPI,0,10,A_Look,S_BSPI_STND2,0,0},	// S_BSPI_STND
    {SPR_BSPI,1,10,A_Look,S_BSPI_STND,0,0},	// S_BSPI_STND2
    {SPR_BSPI,0,20,A_NULL,S_BSPI_RUN1,0,0},	// S_BSPI_SIGHT
    {SPR_BSPI,0,3,A_BabyMetal,S_BSPI_RUN2,0,0},	// S_BSPI_RUN1
    {SPR_BSPI,0,3,A_Chase,S_BSPI_RUN3,0,0},	// S_BSPI_RUN2
    {SPR_BSPI,1,3,A_Chase,S_BSPI_RUN4,0,0},	// S_BSPI_RUN3
    {SPR_BSPI,1,3,A_Chase,S_BSPI_RUN5,0,0},	// S_BSPI_RUN4
    {SPR_BSPI,2,3,A_Chase,S_BSPI_RUN6,0,0},	// S_BSPI_RUN5
    {SPR_BSPI,2,3,A_Chase,S_BSPI_RUN7,0,0},	// S_BSPI_RUN6
    {SPR_BSPI,3,3,A_BabyMetal,S_BSPI_RUN8,0,0},	// S_BSPI_RUN7
    {SPR_BSPI,3,3,A_Chase,S_BSPI_RUN9,0,0},	// S_BSPI_RUN8
    {SPR_BSPI,4,3,A_Chase,S_BSPI_RUN10,0,0},	// S_BSPI_RUN9
    {SPR_BSPI,4,3,A_Chase,S_BSPI_RUN11,0,0},	// S_BSPI_RUN10
    {SPR_BSPI,5,3,A_Chase,S_BSPI_RUN12,0,0},	// S_BSPI_RUN11
    {SPR_BSPI,5,3,A_Chase,S_BSPI_RUN1,0,0},	// S_BSPI_RUN12
    {SPR_BSPI,32768,20,A_FaceTarget,S_BSPI_ATK2,0,0},	// S_BSPI_ATK1
    {SPR_BSPI,32774,4,A_BspiAttack,S_BSPI_ATK3,0,0},	// S_BSPI_ATK2
    {SPR_BSPI,32775,4,A_NULL,S_BSPI_ATK4,0,0},	// S_BSPI_ATK3
    {SPR_BSPI,32775,1,A_SpidRefire,S_BSPI_ATK2,0,0},	// S_BSPI_ATK4
    {SPR_BSPI,8,3,A_NULL,S_BSPI_PAIN2,0,0},	// S_BSPI_PAIN
    {SPR_BSPI,8,3,A_Pain,S_BSPI_RUN1,0,0},	// S_BSPI_PAIN2
    {SPR_BSPI,9,20,A_Scream,S_BSPI_DIE2,0,0},	// S_BSPI_DIE1
    {SPR_BSPI,10,7,A_Fall,S_BSPI_DIE3,0,0},	// S_BSPI_DIE2
    {SPR_BSPI,11,7,A_NULL,S_BSPI_DIE4,0,0},	// S_BSPI_DIE3
    {SPR_BSPI,12,7,A_NULL,S_BSPI_DIE5,0,0},	// S_BSPI_DIE4
    {SPR_BSPI,13,7,A_NULL,S_BSPI_DIE6,0,0},	// S_BSPI_DIE5
    {SPR_BSPI,14,7,A_NULL,S_BSPI_DIE7,0,0},	// S_BSPI_DIE6
    {SPR_BSPI,15,-1,A_BossDeath,S_NULL,0,0},	// S_BSPI_DIE7
    {SPR_BSPI,15,5,A_NULL,S_BSPI_RAISE2,0,0},	// S_BSPI_RAISE1
    {SPR_BSPI,14,5,A_NULL,S_BSPI_RAISE3,0,0},	// S_BSPI_RAISE2
    {SPR_BSPI,13,5,A_NULL,S_BSPI_RAISE4,0,0},	// S_BSPI_RAISE3
    {SPR_BSPI,12,5,A_NULL,S_BSPI_RAISE5,0,0},	// S_BSPI_RAISE4
    {SPR_BSPI,11,5,A_NULL,S_BSPI_RAISE6,0,0},	// S_BSPI_RAISE5
    {SPR_BSPI,10,5,A_NULL,S_BSPI_RAISE7,0,0},	// S_BSPI_RAISE6
    {SPR_BSPI,9,5,A_NULL,S_BSPI_RUN1,0,0},	// S_BSPI_RAISE7
    {SPR_APLS,32768,5,A_NULL,S_ARACH_PLAZ2,0,0},	// S_ARACH_PLAZ
    {SPR_APLS,32769,5,A_NULL,S_ARACH_PLAZ,0,0},	// S_ARACH_PLAZ2
    {SPR_APBX,32768,5,A_NULL,S_ARACH_PLEX2,0,0},	// S_ARACH_PLEX
    {SPR_APBX,32769,5,A_NULL,S_ARACH_PLEX3,0,0},	// S_ARACH_PLEX2
    {SPR_APBX,32770,5,A_NULL,S_ARACH_PLEX4,0,0},	// S_ARACH_PLEX3
    {SPR_APBX,32771,5,A_NULL,S_ARACH_PLEX5,0,0},	// S_ARACH_PLEX4
    {SPR_APBX,32772,5,A_NULL,S_NULL,0,0},	// S_ARACH_PLEX5
    {SPR_CYBR,0,10,A_Look,S_CYBER_STND2,0,0},	// S_CYBER_STND
    {SPR_CYBR,1,10,A_Look,S_CYBER_STND,0,0},	// S_CYBER_STND2
    {SPR_CYBR,0,3,A_Hoof,S_CYBER_RUN2,0,0},	// S_CYBER_RUN1
    {SPR_CYBR,0,3,A_Chase,S_CYBER_RUN3,0,0},	// S_CYBER_RUN2
    {SPR_CYBR,1,3,A_Chase,S_CYBER_RUN4,0,0},	// S_CYBER_RUN3
    {SPR_CYBR,1,3,A_Chase,S_CYBER_RUN5,0,0},	// S_CYBER_RUN4
    {SPR_CYBR,2,3,A_Chase,S_CYBER_RUN6,0,0},	// S_CYBER_RUN5
    {SPR_CYBR,2,3,A_Chase,S_CYBER_RUN7,0,0},	// S_CYBER_RUN6
    {SPR_CYBR,3,3,A_Metal,S_CYBER_RUN8,0,0},	// S_CYBER_RUN7
    {SPR_CYBR,3,3,A_Chase,S_CYBER_RUN1,0,0},	// S_CYBER_RUN8
    {SPR_CYBR,4,6,A_FaceTarget,S_CYBER_ATK2,0,0},	// S_CYBER_ATK1
    {SPR_CYBR,5,12,A_CyberAttack,S_CYBER_ATK3,0,0},	// S_CYBER_ATK2
    {SPR_CYBR,4,12,A_FaceTarget,S_CYBER_ATK4,0,0},	// S_CYBER_ATK3
    {SPR_CYBR,5,12,A_CyberAttack,S_CYBER_ATK5,0,0},	// S_CYBER_ATK4
    {SPR_CYBR,4,12,A_FaceTarget,S_CYBER_ATK6,0,0},	// S_CYBER_ATK5
    {SPR_CYBR,5,12,A_CyberAttack,S_CYBER_RUN1,0,0},	// S_CYBER_ATK6
    {SPR_CYBR,6,10,A_Pain,S_CYBER_RUN1,0,0},	// S_CYBER_PAIN
    {SPR_CYBR,7,10,A_NULL,S_CYBER_DIE2,0,0},	// S_CYBER_DIE1
    {SPR_CYBR,8,10,A_Scream,S_CYBER_DIE3,0,0},	// S_CYBER_DIE2
    {SPR_CYBR,9,10,A_NULL,S_CYBER_DIE4,0,0},	// S_CYBER_DIE3
    {SPR_CYBR,10,10,A_NULL,S_CYBER_DIE5,0,0},	// S_CYBER_DIE4
    {SPR_CYBR,11,10,A_NULL,S_CYBER_DIE6,0,0},	// S_CYBER_DIE5
    {SPR_CYBR,12,10,A_Fall,S_CYBER_DIE7,0,0},	// S_CYBER_DIE6
    {SPR_CYBR,13,10,A_NULL,S_CYBER_DIE8,0,0},	// S_CYBER_DIE7
    {SPR_CYBR,14,10,A_NULL,S_CYBER_DIE9,0,0},	// S_CYBER_DIE8
    {SPR_CYBR,15,30,A_NULL,S_CYBER_DIE10,0,0},	// S_CYBER_DIE9
    {SPR_CYBR,15,-1,A_BossDeath,S_NULL,0,0},	// S_CYBER_DIE10
    {SPR_PAIN,0,10,A_Look,S_PAIN_STND,0,0},	// S_PAIN_STND
    {SPR_PAIN,0,3,A_Chase,S_PAIN_RUN2,0,0},	// S_PAIN_RUN1
    {SPR_PAIN,0,3,A_Chase,S_PAIN_RUN3,0,0},	// S_PAIN_RUN2
    {SPR_PAIN,1,3,A_Chase,S_PAIN_RUN4,0,0},	// S_PAIN_RUN3
    {SPR_PAIN,1,3,A_Chase,S_PAIN_RUN5,0,0},	// S_PAIN_RUN4
    {SPR_PAIN,2,3,A_Chase,S_PAIN_RUN6,0,0},	// S_PAIN_RUN5
    {SPR_PAIN,2,3,A_Chase,S_PAIN_RUN1,0,0},	// S_PAIN_RUN6
    {SPR_PAIN,3,5,A_FaceTarget,S_PAIN_ATK2,0,0},	// S_PAIN_ATK1
    {SPR_PAIN,4,5,A_FaceTarget,S_PAIN_ATK3,0,0},	// S_PAIN_ATK2
    {SPR_PAIN,32773,5,A_FaceTarget,S_PAIN_ATK4,0,0},	// S_PAIN_ATK3
    {SPR_PAIN,32773,0,A_PainAttack,S_PAIN_RUN1,0,0},	// S_PAIN_ATK4
    {SPR_PAIN,6,6,A_NULL,S_PAIN_PAIN2,0,0},	// S_PAIN_PAIN
    {SPR_PAIN,6,6,A_Pain,S_PAIN_RUN1,0,0},	// S_PAIN_PAIN2
    {SPR_PAIN,32775,8,A_NULL,S_PAIN_DIE2,0,0},	// S_PAIN_DIE1
    {SPR_PAIN,32776,8,A_Scream,S_PAIN_DIE3,0,0},	// S_PAIN_DIE2
    {SPR_PAIN,32777,8,A_NULL,S_PAIN_DIE4,0,0},	// S_PAIN_DIE3
    {SPR_PAIN,32778,8,A_NULL,S_PAIN_DIE5,0,0},	// S_PAIN_DIE4
    {SPR_PAIN,32779,8,A_PainDie,S_PAIN_DIE6,0,0},	// S_PAIN_DIE5
    {SPR_PAIN,32780,8,A_NULL,S_NULL,0,0},	// S_PAIN_DIE6
    {SPR_PAIN,12,8,A_NULL,S_PAIN_RAISE2,0,0},	// S_PAIN_RAISE1
    {SPR_PAIN,11,8,A_NULL,S_PAIN_RAISE3,0,0},	// S_PAIN_RAISE2
    {SPR_PAIN,10,8,A_NULL,S_PAIN_RAISE4,0,0},	// S_PAIN_RAISE3
    {SPR_PAIN,9,8,A_NULL,S_PAIN_RAISE5,0,0},	// S_PAIN_RAISE4
    {SPR_PAIN,8,8,A_NULL,S_PAIN_RAISE6,0,0},	// S_PAIN_RAISE5
    {SPR_PAIN,7,8,A_NULL,S_PAIN_RUN1,0,0},	// S_PAIN_RAISE6
    {SPR_SSWV,0,10,A_Look,S_SSWV_STND2,0,0},	// S_SSWV_STND
    {SPR_SSWV,1,10,A_Look,S_SSWV_STND,0,0},	// S_SSWV_STND2
    {SPR_SSWV,0,3,A_Chase,S_SSWV_RUN2,0,0},	// S_SSWV_RUN1
    {SPR_SSWV,0,3,A_Chase,S_SSWV_RUN3,0,0},	// S_SSWV_RUN2
    {SPR_SSWV,1,3,A_Chase,S_SSWV_RUN4,0,0},	// S_SSWV_RUN3
    {SPR_SSWV,1,3,A_Chase,S_SSWV_RUN5,0,0},	// S_SSWV_RUN4
    {SPR_SSWV,2,3,A_Chase,S_SSWV_RUN6,0,0},	// S_SSWV_RUN5
    {SPR_SSWV,2,3,A_Chase,S_SSWV_RUN7,0,0},	// S_SSWV_RUN6
    {SPR_SSWV,3,3,A_Chase,S_SSWV_RUN8,0,0},	// S_SSWV_RUN7
    {SPR_SSWV,3,3,A_Chase,S_SSWV_RUN1,0,0},	// S_SSWV_RUN8
    {SPR_SSWV,4,10,A_FaceTarget,S_SSWV_ATK2,0,0},	// S_SSWV_ATK1
    {SPR_SSWV,5,10,A_FaceTarget,S_SSWV_ATK3,0,0},	// S_SSWV_ATK2
    {SPR_SSWV,32774,4,A_CPosAttack,S_SSWV_ATK4,0,0},	// S_SSWV_ATK3
    {SPR_SSWV,5,6,A_FaceTarget,S_SSWV_ATK5,0,0},	// S_SSWV_ATK4
    {SPR_SSWV,32774,4,A_CPosAttack,S_SSWV_ATK6,0,0},	// S_SSWV_ATK5
    {SPR_SSWV,5,1,A_CPosRefire,S_SSWV_ATK2,0,0},	// S_SSWV_ATK6
    {SPR_SSWV,7,3,A_NULL,S_SSWV_PAIN2,0,0},	// S_SSWV_PAIN
    {SPR_SSWV,7,3,A_Pain,S_SSWV_RUN1,0,0},	// S_SSWV_PAIN2
    {SPR_SSWV,8,5,A_NULL,S_SSWV_DIE2,0,0},	// S_SSWV_DIE1
    {SPR_SSWV,9,5,A_Scream,S_SSWV_DIE3,0,0},	// S_SSWV_DIE2
    {SPR_SSWV,10,5,A_Fall,S_SSWV_DIE4,0,0},	// S_SSWV_DIE3
    {SPR_SSWV,11,5,A_NULL,S_SSWV_DIE5,0,0},	// S_SSWV_DIE4
    {SPR_SSWV,12,-1,A_NULL,S_NULL,0,0},	// S_SSWV_DIE5
    {SPR_SSWV,13,5,A_NULL,S_SSWV_XDIE2,0,0},	// S_SSWV_XDIE1
    {SPR_SSWV,14,5,A_XScream,S_SSWV_XDIE3,0,0},	// S_SSWV_XDIE2
    {SPR_SSWV,15,5,A_Fall,S_SSWV_XDIE4,0,0},	// S_SSWV_XDIE3
    {SPR_SSWV,16,5,A_NULL,S_SSWV_XDIE5,0,0},	// S_SSWV_XDIE4
    {SPR_SSWV,17,5,A_NULL,S_SSWV_XDIE6,0,0},	// S_SSWV_XDIE5
    {SPR_SSWV,18,5,A_NULL,S_SSWV_XDIE7,0,0},	// S_SSWV_XDIE6
    {SPR_SSWV,19,5,A_NULL,S_SSWV_XDIE8,0,0},	// S_SSWV_XDIE7
    {SPR_SSWV,20,5,A_NULL,S_SSWV_XDIE9,0,0},	// S_SSWV_XDIE8
    {SPR_SSWV,21,-1,A_NULL,S_NULL,0,0},	// S_SSWV_XDIE9
    {SPR_SSWV,12,5,A_NULL,S_SSWV_RAISE2,0,0},	// S_SSWV_RAISE1
    {SPR_SSWV,11,5,A_NULL,S_SSWV_RAISE3,0,0},	// S_SSWV_RAISE2
    {SPR_SSWV,10,5,A_NULL,S_SSWV_RAISE4,0,0},	// S_SSWV_RAISE3
    {SPR_SSWV,9,5,A_NULL,S_SSWV_RAISE5,0,0},	// S_SSWV_RAISE4
    {SPR_SSWV,8,5,A_NULL,S_SSWV_RUN1,0,0},	// S_SSWV_RAISE5
    {SPR_KEEN,0,-1,A_NULL,S_KEENSTND,0,0},	// S_KEENSTND
    {SPR_KEEN,0,6,A_NULL,S_COMMKEEN2,0,0},	// S_COMMKEEN
    {SPR_KEEN,1,6,A_NULL,S_COMMKEEN3,0,0},	// S_COMMKEEN2
    {SPR_KEEN,2,6,A_Scream,S_COMMKEEN4,0,0},	// S_COMMKEEN3
    {SPR_KEEN,3,6,A_NULL,S_COMMKEEN5,0,0},	// S_COMMKEEN4
    {SPR_KEEN,4,6,A_NULL,S_COMMKEEN6,0,0},	// S_COMMKEEN5
    {SPR_KEEN,5,6,A_NULL,S_COMMKEEN7,0,0},	// S_COMMKEEN6
    {SPR_KEEN,6,6,A_NULL,S_COMMKEEN8,0,0},	// S_COMMKEEN7
    {SPR_KEEN,7,6,A_NULL,S_COMMKEEN9,0,0},	// S_COMMKEEN8
    {SPR_KEEN,8,6,A_NULL,S_COMMKEEN10,0,0},	// S_COMMKEEN9
    {SPR_KEEN,9,6,A_NULL,S_COMMKEEN11,0,0},	// S_COMMKEEN10
    {SPR_KEEN,10,6,A_KeenDie,S_COMMKEEN12,0,0},// S_COMMKEEN11
    {SPR_KEEN,11,-1,A_NULL,S_NULL,0,0},		// S_COMMKEEN12
    {SPR_KEEN,12,4,A_NULL,S_KEENPAIN2,0,0},	// S_KEENPAIN
    {SPR_KEEN,12,8,A_Pain,S_KEENSTND,0,0},	// S_KEENPAIN2
    {SPR_BBRN,0,-1,A_NULL,S_NULL,0,0},		// S_BRAIN
    {SPR_BBRN,1,36,A_BrainPain,S_BRAIN,0,0},	// S_BRAIN_PAIN
    {SPR_BBRN,0,100,A_BrainScream,S_BRAIN_DIE2,0,0},	// S_BRAIN_DIE1
    {SPR_BBRN,0,10,A_NULL,S_BRAIN_DIE3,0,0},	// S_BRAIN_DIE2
    {SPR_BBRN,0,10,A_NULL,S_BRAIN_DIE4,0,0},	// S_BRAIN_DIE3
    {SPR_BBRN,0,-1,A_BrainDie,S_NULL,0,0},	// S_BRAIN_DIE4
    {SPR_SSWV,0,10,A_Look,S_BRAINEYE,0,0},	// S_BRAINEYE
    {SPR_SSWV,0,181,A_BrainAwake,S_BRAINEYE1,0,0},	// S_BRAINEYESEE
    {SPR_SSWV,0,150,A_BrainSpit,S_BRAINEYE1,0,0},	// S_BRAINEYE1
    {SPR_BOSF,32768,3,A_SpawnSound,S_SPAWN2,0,0},	// S_SPAWN1
    {SPR_BOSF,32769,3,A_SpawnFly,S_SPAWN3,0,0},	// S_SPAWN2
    {SPR_BOSF,32770,3,A_SpawnFly,S_SPAWN4,0,0},	// S_SPAWN3
    {SPR_BOSF,32771,3,A_SpawnFly,S_SPAWN1,0,0},	// S_SPAWN4
    {SPR_FIRE,32768,4,A_Fire,S_SPAWNFIRE2,0,0},	// S_SPAWNFIRE1
    {SPR_FIRE,32769,4,A_Fire,S_SPAWNFIRE3,0,0},	// S_SPAWNFIRE2
    {SPR_FIRE,32770,4,A_Fire,S_SPAWNFIRE4,0,0},	// S_SPAWNFIRE3
    {SPR_FIRE,32771,4,A_Fire,S_SPAWNFIRE5,0,0},	// S_SPAWNFIRE4
    {SPR_FIRE,32772,4,A_Fire,S_SPAWNFIRE6,0,0},	// S_SPAWNFIRE5
    {SPR_FIRE,32773,4,A_Fire,S_SPAWNFIRE7,0,0},	// S_SPAWNFIRE6
    {SPR_FIRE,32774,4,A_Fire,S_SPAWNFIRE8,0,0},	// S_SPAWNFIRE7
    {SPR_FIRE,32775,4,A_CubeSpawn,S_NULL,0,0},		// S_SPAWNFIRE8  /* MOD */
    {SPR_MISL,32769,10,A_NULL,S_BRAINEXPLODE2,0,0},	// S_BRAINEXPLODE1
    {SPR_MISL,32770,10,A_NULL,S_BRAINEXPLODE3,0,0},	// S_BRAINEXPLODE2
    {SPR_MISL,32771,10,A_BrainExplode,S_NULL,0,0},	// S_BRAINEXPLODE3
    {SPR_ARM1,0,6,A_NULL,S_ARM1A,0,0},	// S_ARM1
    {SPR_ARM1,32769,7,A_NULL,S_ARM1,0,0},	// S_ARM1A
    {SPR_ARM2,0,6,A_NULL,S_ARM2A,0,0},	// S_ARM2
    {SPR_ARM2,32769,6,A_NULL,S_ARM2,0,0},	// S_ARM2A
    {SPR_BAR1,0,6,A_NULL,S_BAR2,0,0},	// S_BAR1
    {SPR_BAR1,1,6,A_NULL,S_BAR1,0,0},	// S_BAR2
    {SPR_BEXP,32768,5,A_NULL,S_BEXP2,0,0},	// S_BEXP
    {SPR_BEXP,32769,5,A_Scream,S_BEXP3,0,0},	// S_BEXP2
    {SPR_BEXP,32770,5,A_NULL,S_BEXP4,0,0},	// S_BEXP3
    {SPR_BEXP,32771,10,A_Explode,S_BEXP5,0,0},	// S_BEXP4
    {SPR_BEXP,32772,10,A_NULL,S_NULL,0,0},	// S_BEXP5
    {SPR_FCAN,32768,4,A_NULL,S_BBAR2,0,0},	// S_BBAR1
    {SPR_FCAN,32769,4,A_NULL,S_BBAR3,0,0},	// S_BBAR2
    {SPR_FCAN,32770,4,A_NULL,S_BBAR1,0,0},	// S_BBAR3
    {SPR_BON1,0,6,A_NULL,S_BON1A,0,0},	// S_BON1
    {SPR_BON1,1,6,A_NULL,S_BON1B,0,0},	// S_BON1A
    {SPR_BON1,2,6,A_NULL,S_BON1C,0,0},	// S_BON1B
    {SPR_BON1,3,6,A_NULL,S_BON1D,0,0},	// S_BON1C
    {SPR_BON1,2,6,A_NULL,S_BON1E,0,0},	// S_BON1D
    {SPR_BON1,1,6,A_NULL,S_BON1,0,0},	// S_BON1E
    {SPR_BON2,0,6,A_NULL,S_BON2A,0,0},	// S_BON2
    {SPR_BON2,1,6,A_NULL,S_BON2B,0,0},	// S_BON2A
    {SPR_BON2,2,6,A_NULL,S_BON2C,0,0},	// S_BON2B
    {SPR_BON2,3,6,A_NULL,S_BON2D,0,0},	// S_BON2C
    {SPR_BON2,2,6,A_NULL,S_BON2E,0,0},	// S_BON2D
    {SPR_BON2,1,6,A_NULL,S_BON2,0,0},	// S_BON2E
    {SPR_BKEY,0,10,A_NULL,S_BKEY2,0,0},	// S_BKEY
    {SPR_BKEY,32769,10,A_NULL,S_BKEY,0,0},	// S_BKEY2
    {SPR_RKEY,0,10,A_NULL,S_RKEY2,0,0},	// S_RKEY
    {SPR_RKEY,32769,10,A_NULL,S_RKEY,0,0},	// S_RKEY2
    {SPR_YKEY,0,10,A_NULL,S_YKEY2,0,0},	// S_YKEY
    {SPR_YKEY,32769,10,A_NULL,S_YKEY,0,0},	// S_YKEY2
    {SPR_BSKU,0,10,A_NULL,S_BSKULL2,0,0},	// S_BSKULL
    {SPR_BSKU,32769,10,A_NULL,S_BSKULL,0,0},	// S_BSKULL2
    {SPR_RSKU,0,10,A_NULL,S_RSKULL2,0,0},	// S_RSKULL
    {SPR_RSKU,32769,10,A_NULL,S_RSKULL,0,0},	// S_RSKULL2
    {SPR_YSKU,0,10,A_NULL,S_YSKULL2,0,0},	// S_YSKULL
    {SPR_YSKU,32769,10,A_NULL,S_YSKULL,0,0},	// S_YSKULL2
    {SPR_STIM,0,-1,A_NULL,S_NULL,0,0},	// S_STIM
    {SPR_MEDI,0,-1,A_NULL,S_NULL,0,0},	// S_MEDI
    {SPR_SOUL,32768,6,A_NULL,S_SOUL2,0,0},	// S_SOUL
    {SPR_SOUL,32769,6,A_NULL,S_SOUL3,0,0},	// S_SOUL2
    {SPR_SOUL,32770,6,A_NULL,S_SOUL4,0,0},	// S_SOUL3
    {SPR_SOUL,32771,6,A_NULL,S_SOUL5,0,0},	// S_SOUL4
    {SPR_SOUL,32770,6,A_NULL,S_SOUL6,0,0},	// S_SOUL5
    {SPR_SOUL,32769,6,A_NULL,S_SOUL,0,0},	// S_SOUL6
    {SPR_PINV,32768,6,A_NULL,S_PINV2,0,0},	// S_PINV
    {SPR_PINV,32769,6,A_NULL,S_PINV3,0,0},	// S_PINV2
    {SPR_PINV,32770,6,A_NULL,S_PINV4,0,0},	// S_PINV3
    {SPR_PINV,32771,6,A_NULL,S_PINV,0,0},	// S_PINV4
    {SPR_PSTR,32768,-1,A_NULL,S_NULL,0,0},	// S_PSTR
    {SPR_PINS,32768,6,A_NULL,S_PINS2,0,0},	// S_PINS
    {SPR_PINS,32769,6,A_NULL,S_PINS3,0,0},	// S_PINS2
    {SPR_PINS,32770,6,A_NULL,S_PINS4,0,0},	// S_PINS3
    {SPR_PINS,32771,6,A_NULL,S_PINS,0,0},	// S_PINS4
    {SPR_MEGA,32768,6,A_NULL,S_MEGA2,0,0},	// S_MEGA
    {SPR_MEGA,32769,6,A_NULL,S_MEGA3,0,0},	// S_MEGA2
    {SPR_MEGA,32770,6,A_NULL,S_MEGA4,0,0},	// S_MEGA3
    {SPR_MEGA,32771,6,A_NULL,S_MEGA,0,0},	// S_MEGA4
    {SPR_SUIT,32768,-1,A_NULL,S_NULL,0,0},	// S_SUIT
    {SPR_PMAP,32768,6,A_NULL,S_PMAP2,0,0},	// S_PMAP
    {SPR_PMAP,32769,6,A_NULL,S_PMAP3,0,0},	// S_PMAP2
    {SPR_PMAP,32770,6,A_NULL,S_PMAP4,0,0},	// S_PMAP3
    {SPR_PMAP,32771,6,A_NULL,S_PMAP5,0,0},	// S_PMAP4
    {SPR_PMAP,32770,6,A_NULL,S_PMAP6,0,0},	// S_PMAP5
    {SPR_PMAP,32769,6,A_NULL,S_PMAP,0,0},	// S_PMAP6
    {SPR_PVIS,32768,6,A_NULL,S_PVIS2,0,0},	// S_PVIS
    {SPR_PVIS,1,6,A_NULL,S_PVIS,0,0},	// S_PVIS2
    {SPR_CLIP,0,-1,A_NULL,S_NULL,0,0},	// S_CLIP
    {SPR_AMMO,0,-1,A_NULL,S_NULL,0,0},	// S_AMMO
    {SPR_ROCK,0,-1,A_NULL,S_NULL,0,0},	// S_ROCK
    {SPR_BROK,0,-1,A_NULL,S_NULL,0,0},	// S_BROK
    {SPR_CELL,0,-1,A_NULL,S_NULL,0,0},	// S_CELL
    {SPR_CELP,0,-1,A_NULL,S_NULL,0,0},	// S_CELP
    {SPR_SHEL,0,-1,A_NULL,S_NULL,0,0},	// S_SHEL
    {SPR_SBOX,0,-1,A_NULL,S_NULL,0,0},	// S_SBOX
    {SPR_BPAK,0,-1,A_NULL,S_NULL,0,0},	// S_BPAK
    {SPR_BFUG,0,-1,A_NULL,S_NULL,0,0},	// S_BFUG
    {SPR_MGUN,0,-1,A_NULL,S_NULL,0,0},	// S_MGUN
    {SPR_CSAW,0,-1,A_NULL,S_NULL,0,0},	// S_CSAW
    {SPR_LAUN,0,-1,A_NULL,S_NULL,0,0},	// S_LAUN
    {SPR_PLAS,0,-1,A_NULL,S_NULL,0,0},	// S_PLAS
    {SPR_SHOT,0,-1,A_NULL,S_NULL,0,0},	// S_SHOT
    {SPR_SGN2,0,-1,A_NULL,S_NULL,0,0},	// S_SHOT2
    {SPR_COLU,32768,-1,A_NULL,S_NULL,0,0},	// S_COLU
    {SPR_SMT2,0,-1,A_NULL,S_NULL,0,0},	// S_STALAG
    {SPR_GOR1,0,10,A_NULL,S_BLOODYTWITCH2,0,0},	// S_BLOODYTWITCH
    {SPR_GOR1,1,15,A_NULL,S_BLOODYTWITCH3,0,0},	// S_BLOODYTWITCH2
    {SPR_GOR1,2,8,A_NULL,S_BLOODYTWITCH4,0,0},	// S_BLOODYTWITCH3
    {SPR_GOR1,1,6,A_NULL,S_BLOODYTWITCH,0,0},	// S_BLOODYTWITCH4
    {SPR_PLAY,13,-1,A_NULL,S_NULL,0,0},	// S_DEADTORSO
    {SPR_PLAY,18,-1,A_NULL,S_NULL,0,0},	// S_DEADBOTTOM
    {SPR_POL2,0,-1,A_NULL,S_NULL,0,0},	// S_HEADSONSTICK
    {SPR_POL5,0,-1,A_NULL,S_NULL,0,0},	// S_GIBS
    {SPR_POL4,0,-1,A_NULL,S_NULL,0,0},	// S_HEADONASTICK
    {SPR_POL3,32768,6,A_NULL,S_HEADCANDLES2,0,0},	// S_HEADCANDLES
    {SPR_POL3,32769,6,A_NULL,S_HEADCANDLES,0,0},	// S_HEADCANDLES2
    {SPR_POL1,0,-1,A_NULL,S_NULL,0,0},	// S_DEADSTICK
    {SPR_POL6,0,6,A_NULL,S_LIVESTICK2,0,0},	// S_LIVESTICK
    {SPR_POL6,1,8,A_NULL,S_LIVESTICK,0,0},	// S_LIVESTICK2
    {SPR_GOR2,0,-1,A_NULL,S_NULL,0,0},	// S_MEAT2
    {SPR_GOR3,0,-1,A_NULL,S_NULL,0,0},	// S_MEAT3
    {SPR_GOR4,0,-1,A_NULL,S_NULL,0,0},	// S_MEAT4
    {SPR_GOR5,0,-1,A_NULL,S_NULL,0,0},	// S_MEAT5
    {SPR_SMIT,0,-1,A_NULL,S_NULL,0,0},	// S_STALAGTITE
    {SPR_COL1,0,-1,A_NULL,S_NULL,0,0},	// S_TALLGRNCOL
    {SPR_COL2,0,-1,A_NULL,S_NULL,0,0},	// S_SHRTGRNCOL
    {SPR_COL3,0,-1,A_NULL,S_NULL,0,0},	// S_TALLREDCOL
    {SPR_COL4,0,-1,A_NULL,S_NULL,0,0},	// S_SHRTREDCOL
    {SPR_CAND,32768,-1,A_NULL,S_NULL,0,0},	// S_CANDLESTIK
    {SPR_CBRA,32768,-1,A_NULL,S_NULL,0,0},	// S_CANDELABRA
    {SPR_COL6,0,-1,A_NULL,S_NULL,0,0},	// S_SKULLCOL
    {SPR_TRE1,0,-1,A_NULL,S_NULL,0,0},	// S_TORCHTREE
    {SPR_TRE2,0,-1,A_NULL,S_NULL,0,0},	// S_BIGTREE
    {SPR_ELEC,0,-1,A_NULL,S_NULL,0,0},	// S_TECHPILLAR
    {SPR_CEYE,32768,6,A_NULL,S_EVILEYE2,0,0},	// S_EVILEYE
    {SPR_CEYE,32769,6,A_NULL,S_EVILEYE3,0,0},	// S_EVILEYE2
    {SPR_CEYE,32770,6,A_NULL,S_EVILEYE4,0,0},	// S_EVILEYE3
    {SPR_CEYE,32769,6,A_NULL,S_EVILEYE,0,0},	// S_EVILEYE4
    {SPR_FSKU,32768,6,A_NULL,S_FLOATSKULL2,0,0},	// S_FLOATSKULL
    {SPR_FSKU,32769,6,A_NULL,S_FLOATSKULL3,0,0},	// S_FLOATSKULL2
    {SPR_FSKU,32770,6,A_NULL,S_FLOATSKULL,0,0},	// S_FLOATSKULL3
    {SPR_COL5,0,14,A_NULL,S_HEARTCOL2,0,0},	// S_HEARTCOL
    {SPR_COL5,1,14,A_NULL,S_HEARTCOL,0,0},	// S_HEARTCOL2
    {SPR_TBLU,32768,4,A_NULL,S_BLUETORCH2,0,0},	// S_BLUETORCH
    {SPR_TBLU,32769,4,A_NULL,S_BLUETORCH3,0,0},	// S_BLUETORCH2
    {SPR_TBLU,32770,4,A_NULL,S_BLUETORCH4,0,0},	// S_BLUETORCH3
    {SPR_TBLU,32771,4,A_NULL,S_BLUETORCH,0,0},	// S_BLUETORCH4
    {SPR_TGRN,32768,4,A_NULL,S_GREENTORCH2,0,0},	// S_GREENTORCH
    {SPR_TGRN,32769,4,A_NULL,S_GREENTORCH3,0,0},	// S_GREENTORCH2
    {SPR_TGRN,32770,4,A_NULL,S_GREENTORCH4,0,0},	// S_GREENTORCH3
    {SPR_TGRN,32771,4,A_NULL,S_GREENTORCH,0,0},	// S_GREENTORCH4
    {SPR_TRED,32768,4,A_NULL,S_REDTORCH2,0,0},	// S_REDTORCH
    {SPR_TRED,32769,4,A_NULL,S_REDTORCH3,0,0},	// S_REDTORCH2
    {SPR_TRED,32770,4,A_NULL,S_REDTORCH4,0,0},	// S_REDTORCH3
    {SPR_TRED,32771,4,A_NULL,S_REDTORCH,0,0},	// S_REDTORCH4
    {SPR_SMBT,32768,4,A_NULL,S_BTORCHSHRT2,0,0},	// S_BTORCHSHRT
    {SPR_SMBT,32769,4,A_NULL,S_BTORCHSHRT3,0,0},	// S_BTORCHSHRT2
    {SPR_SMBT,32770,4,A_NULL,S_BTORCHSHRT4,0,0},	// S_BTORCHSHRT3
    {SPR_SMBT,32771,4,A_NULL,S_BTORCHSHRT,0,0},	// S_BTORCHSHRT4
    {SPR_SMGT,32768,4,A_NULL,S_GTORCHSHRT2,0,0},	// S_GTORCHSHRT
    {SPR_SMGT,32769,4,A_NULL,S_GTORCHSHRT3,0,0},	// S_GTORCHSHRT2
    {SPR_SMGT,32770,4,A_NULL,S_GTORCHSHRT4,0,0},	// S_GTORCHSHRT3
    {SPR_SMGT,32771,4,A_NULL,S_GTORCHSHRT,0,0},	// S_GTORCHSHRT4
    {SPR_SMRT,32768,4,A_NULL,S_RTORCHSHRT2,0,0},	// S_RTORCHSHRT
    {SPR_SMRT,32769,4,A_NULL,S_RTORCHSHRT3,0,0},	// S_RTORCHSHRT2
    {SPR_SMRT,32770,4,A_NULL,S_RTORCHSHRT4,0,0},	// S_RTORCHSHRT3
    {SPR_SMRT,32771,4,A_NULL,S_RTORCHSHRT,0,0},	// S_RTORCHSHRT4
    {SPR_HDB1,0,-1,A_NULL,S_NULL,0,0},	// S_HANGNOGUTS
    {SPR_HDB2,0,-1,A_NULL,S_NULL,0,0},	// S_HANGBNOBRAIN
    {SPR_HDB3,0,-1,A_NULL,S_NULL,0,0},	// S_HANGTLOOKDN
    {SPR_HDB4,0,-1,A_NULL,S_NULL,0,0},	// S_HANGTSKULL
    {SPR_HDB5,0,-1,A_NULL,S_NULL,0,0},	// S_HANGTLOOKUP
    {SPR_HDB6,0,-1,A_NULL,S_NULL,0,0},	// S_HANGTNOBRAIN
    {SPR_POB1,0,-1,A_NULL,S_NULL,0,0},	// S_COLONGIBS
    {SPR_POB2,0,-1,A_NULL,S_NULL,0,0},	// S_SMALLPOOL
    {SPR_BRS1,0,-1,A_NULL,S_NULL,0,0},		// S_BRAINSTEM
    {SPR_TLMP,32768,4,A_NULL,S_TECHLAMP2,0,0},	// S_TECHLAMP
    {SPR_TLMP,32769,4,A_NULL,S_TECHLAMP3,0,0},	// S_TECHLAMP2
    {SPR_TLMP,32770,4,A_NULL,S_TECHLAMP4,0,0},	// S_TECHLAMP3
    {SPR_TLMP,32771,4,A_NULL,S_TECHLAMP,0,0},	// S_TECHLAMP4
    {SPR_TLP2,32768,4,A_NULL,S_TECH2LAMP2,0,0},	// S_TECH2LAMP
    {SPR_TLP2,32769,4,A_NULL,S_TECH2LAMP3,0,0},	// S_TECH2LAMP2
    {SPR_TLP2,32770,4,A_NULL,S_TECH2LAMP4,0,0},	// S_TECH2LAMP3
    {SPR_TLP2,32771,4,A_NULL,S_TECH2LAMP,0,0},	// S_TECH2LAMP4

	// BOOM and MBF...
	{SPR_TNT1,0,-1,A_NULL,S_TNT1,0,0},            // S_TNT1
	{SPR_MISL,32768,1000,A_Die,S_GRENADE},      // S_GRENADE
	{SPR_MISL,32769,4,A_Scream,S_DETONATE2},    // S_DETONATE
	{SPR_MISL,32770,6,A_Detonate,S_DETONATE3},  // S_DETONATE2
	{SPR_MISL,32771,10,A_NULL,S_NULL},            // S_DETONATE3
	{SPR_DOGS,0,10,A_Look,S_DOGS_STND2},  // S_DOGS_STND
	{SPR_DOGS,1,10,A_Look,S_DOGS_STND}, // S_DOGS_STND2
	{SPR_DOGS,0,2,A_Chase,S_DOGS_RUN2}, // S_DOGS_RUN1
	{SPR_DOGS,0,2,A_Chase,S_DOGS_RUN3}, // S_DOGS_RUN2
	{SPR_DOGS,1,2,A_Chase,S_DOGS_RUN4}, // S_DOGS_RUN3
	{SPR_DOGS,1,2,A_Chase,S_DOGS_RUN5}, // S_DOGS_RUN4
	{SPR_DOGS,2,2,A_Chase,S_DOGS_RUN6}, // S_DOGS_RUN5
	{SPR_DOGS,2,2,A_Chase,S_DOGS_RUN7}, // S_DOGS_RUN6
	{SPR_DOGS,3,2,A_Chase,S_DOGS_RUN8}, // S_DOGS_RUN7
	{SPR_DOGS,3,2,A_Chase,S_DOGS_RUN1}, // S_DOGS_RUN8
	{SPR_DOGS,4,8,A_FaceTarget,S_DOGS_ATK2},  // S_DOGS_ATK1
	{SPR_DOGS,5,8,A_FaceTarget,S_DOGS_ATK3},  // S_DOGS_ATK2
	{SPR_DOGS,6,8,A_SargAttack,S_DOGS_RUN1},  // S_DOGS_ATK3
	{SPR_DOGS,7,2,A_NULL,S_DOGS_PAIN2}, // S_DOGS_PAIN
	{SPR_DOGS,7,2,A_Pain,S_DOGS_RUN1},  // S_DOGS_PAIN2
	{SPR_DOGS,8,8,A_NULL,S_DOGS_DIE2},  // S_DOGS_DIE1
	{SPR_DOGS,9,8,A_Scream,S_DOGS_DIE3},  // S_DOGS_DIE2
	{SPR_DOGS,10,4,A_NULL,S_DOGS_DIE4}, // S_DOGS_DIE3
	{SPR_DOGS,11,4,A_Fall,S_DOGS_DIE5}, // S_DOGS_DIE4
	{SPR_DOGS,12,4,A_NULL,S_DOGS_DIE6}, // S_DOGS_DIE5
	{SPR_DOGS,13,-1,A_NULL,S_NULL}, // S_DOGS_DIE6
	{SPR_DOGS,13,5,A_NULL,S_DOGS_RAISE2}, // S_DOGS_RAISE1
	{SPR_DOGS,12,5,A_NULL,S_DOGS_RAISE3}, // S_DOGS_RAISE2
	{SPR_DOGS,11,5,A_NULL,S_DOGS_RAISE4}, // S_DOGS_RAISE3
	{SPR_DOGS,10,5,A_NULL,S_DOGS_RAISE5}, // S_DOGS_RAISE4
	{SPR_DOGS,9,5,A_NULL,S_DOGS_RAISE6},  // S_DOGS_RAISE5
	{SPR_DOGS,8,5,A_NULL,S_DOGS_RUN1},  // S_DOGS_RAISE6
	{SPR_MISL,32769,8,A_Mushroom,S_EXPLODE2}  // S_MUSHROOM
};


//------------------------------------------------------------------------

#if (DEBUG_RANGES)
const char *state_names[NUMSTATES_BEX] =
{
    "S_NULL",
    "S_LIGHTDONE",
    "S_PUNCH", "S_PUNCHDOWN", "S_PUNCHUP",
    "S_PUNCH1", "S_PUNCH2", "S_PUNCH3", "S_PUNCH4", "S_PUNCH5",
    "S_PISTOL", "S_PISTOLDOWN", "S_PISTOLUP",
    "S_PISTOL1", "S_PISTOL2", "S_PISTOL3", "S_PISTOL4", "S_PISTOLFLASH",
    "S_SGUN", "S_SGUNDOWN", "S_SGUNUP",
    "S_SGUN1", "S_SGUN2", "S_SGUN3", "S_SGUN4", "S_SGUN5", "S_SGUN6",
	"S_SGUN7", "S_SGUN8", "S_SGUN9", "S_SGUNFLASH1", "S_SGUNFLASH2",
    "S_DSGUN", "S_DSGUNDOWN", "S_DSGUNUP",
    "S_DSGUN1", "S_DSGUN2", "S_DSGUN3", "S_DSGUN4", "S_DSGUN5", "S_DSGUN6",
    "S_DSGUN7", "S_DSGUN8", "S_DSGUN9", "S_DSGUN10", "S_DSNR1", "S_DSNR2",
    "S_DSGUNFLASH1", "S_DSGUNFLASH2",
    "S_CHAIN", "S_CHAINDOWN", "S_CHAINUP",
    "S_CHAIN1", "S_CHAIN2", "S_CHAIN3", "S_CHAINFLASH1", "S_CHAINFLASH2",
    "S_MISSILE", "S_MISSILEDOWN", "S_MISSILEUP",
    "S_MISSILE1", "S_MISSILE2", "S_MISSILE3",
    "S_MISSILEFLASH1", "S_MISSILEFLASH2", "S_MISSILEFLASH3", "S_MISSILEFLASH4",
    "S_SAW", "S_SAWB", "S_SAWDOWN", "S_SAWUP",
    "S_SAW1", "S_SAW2", "S_SAW3",
    "S_PLASMA", "S_PLASMADOWN", "S_PLASMAUP",
    "S_PLASMA1", "S_PLASMA2", "S_PLASMAFLASH1", "S_PLASMAFLASH2",
    "S_BFG", "S_BFGDOWN", "S_BFGUP",
    "S_BFG1", "S_BFG2", "S_BFG3", "S_BFG4", "S_BFGFLASH1", "S_BFGFLASH2",
    "S_BLOOD1", "S_BLOOD2", "S_BLOOD3",
    "S_PUFF1", "S_PUFF2", "S_PUFF3", "S_PUFF4",
    "S_TBALL1", "S_TBALL2", "S_TBALLX1", "S_TBALLX2", "S_TBALLX3",
    "S_RBALL1", "S_RBALL2", "S_RBALLX1", "S_RBALLX2", "S_RBALLX3",
    "S_PLASBALL", "S_PLASBALL2",
    "S_PLASEXP", "S_PLASEXP2", "S_PLASEXP3", "S_PLASEXP4", "S_PLASEXP5",
    "S_ROCKET",
    "S_BFGSHOT", "S_BFGSHOT2", "S_BFGLAND", "S_BFGLAND2", "S_BFGLAND3",
    "S_BFGLAND4", "S_BFGLAND5", "S_BFGLAND6", "S_BFGEXP", "S_BFGEXP2",
    "S_BFGEXP3", "S_BFGEXP4",
    "S_EXPLODE1", "S_EXPLODE2", "S_EXPLODE3",
    "S_TFOG", "S_TFOG01", "S_TFOG02", "S_TFOG2", "S_TFOG3", "S_TFOG4",
    "S_TFOG5", "S_TFOG6", "S_TFOG7", "S_TFOG8", "S_TFOG9", "S_TFOG10",
    "S_IFOG", "S_IFOG01", "S_IFOG02", "S_IFOG2", "S_IFOG3", "S_IFOG4", "S_IFOG5",
    "S_PLAY", "S_PLAY_RUN1", "S_PLAY_RUN2", "S_PLAY_RUN3", "S_PLAY_RUN4",
    "S_PLAY_ATK1", "S_PLAY_ATK2", "S_PLAY_PAIN", "S_PLAY_PAIN2",
    "S_PLAY_DIE1", "S_PLAY_DIE2", "S_PLAY_DIE3", "S_PLAY_DIE4", "S_PLAY_DIE5",
    "S_PLAY_DIE6", "S_PLAY_DIE7",
    "S_PLAY_XDIE1", "S_PLAY_XDIE2", "S_PLAY_XDIE3", "S_PLAY_XDIE4",
    "S_PLAY_XDIE5", "S_PLAY_XDIE6", "S_PLAY_XDIE7", "S_PLAY_XDIE8",
    "S_PLAY_XDIE9",
    "S_POSS_STND", "S_POSS_STND2", "S_POSS_RUN1", "S_POSS_RUN2", "S_POSS_RUN3",
    "S_POSS_RUN4", "S_POSS_RUN5", "S_POSS_RUN6", "S_POSS_RUN7", "S_POSS_RUN8",
    "S_POSS_ATK1", "S_POSS_ATK2", "S_POSS_ATK3", "S_POSS_PAIN", "S_POSS_PAIN2",
    "S_POSS_DIE1", "S_POSS_DIE2", "S_POSS_DIE3", "S_POSS_DIE4", "S_POSS_DIE5",
    "S_POSS_XDIE1", "S_POSS_XDIE2", "S_POSS_XDIE3", "S_POSS_XDIE4", "S_POSS_XDIE5",
    "S_POSS_XDIE6", "S_POSS_XDIE7", "S_POSS_XDIE8", "S_POSS_XDIE9", "S_POSS_RAISE1",
    "S_POSS_RAISE2", "S_POSS_RAISE3", "S_POSS_RAISE4",
    "S_SPOS_STND", "S_SPOS_STND2", "S_SPOS_RUN1", "S_SPOS_RUN2", "S_SPOS_RUN3",
    "S_SPOS_RUN4", "S_SPOS_RUN5", "S_SPOS_RUN6", "S_SPOS_RUN7", "S_SPOS_RUN8",
    "S_SPOS_ATK1", "S_SPOS_ATK2", "S_SPOS_ATK3", "S_SPOS_PAIN", "S_SPOS_PAIN2",
    "S_SPOS_DIE1", "S_SPOS_DIE2", "S_SPOS_DIE3", "S_SPOS_DIE4", "S_SPOS_DIE5",
    "S_SPOS_XDIE1", "S_SPOS_XDIE2", "S_SPOS_XDIE3", "S_SPOS_XDIE4", "S_SPOS_XDIE5",
    "S_SPOS_XDIE6", "S_SPOS_XDIE7", "S_SPOS_XDIE8", "S_SPOS_XDIE9", "S_SPOS_RAISE1",
    "S_SPOS_RAISE2", "S_SPOS_RAISE3", "S_SPOS_RAISE4", "S_SPOS_RAISE5",
    "S_VILE_STND", "S_VILE_STND2", "S_VILE_RUN1", "S_VILE_RUN2", "S_VILE_RUN3",
    "S_VILE_RUN4", "S_VILE_RUN5", "S_VILE_RUN6", "S_VILE_RUN7", "S_VILE_RUN8",
    "S_VILE_RUN9", "S_VILE_RUN10", "S_VILE_RUN11", "S_VILE_RUN12", "S_VILE_ATK1",
    "S_VILE_ATK2", "S_VILE_ATK3", "S_VILE_ATK4", "S_VILE_ATK5", "S_VILE_ATK6",
    "S_VILE_ATK7", "S_VILE_ATK8", "S_VILE_ATK9", "S_VILE_ATK10", "S_VILE_ATK11",
    "S_VILE_HEAL1", "S_VILE_HEAL2", "S_VILE_HEAL3", "S_VILE_PAIN", "S_VILE_PAIN2",
    "S_VILE_DIE1", "S_VILE_DIE2", "S_VILE_DIE3", "S_VILE_DIE4", "S_VILE_DIE5",
    "S_VILE_DIE6", "S_VILE_DIE7", "S_VILE_DIE8", "S_VILE_DIE9", "S_VILE_DIE10",
    "S_FIRE1", "S_FIRE2", "S_FIRE3", "S_FIRE4", "S_FIRE5", "S_FIRE6", "S_FIRE7",
    "S_FIRE8", "S_FIRE9", "S_FIRE10", "S_FIRE11", "S_FIRE12", "S_FIRE13", "S_FIRE14",
    "S_FIRE15", "S_FIRE16", "S_FIRE17", "S_FIRE18", "S_FIRE19", "S_FIRE20", "S_FIRE21",
    "S_FIRE22", "S_FIRE23", "S_FIRE24", "S_FIRE25", "S_FIRE26", "S_FIRE27",
    "S_FIRE28", "S_FIRE29", "S_FIRE30",
    "S_SMOKE1", "S_SMOKE2", "S_SMOKE3", "S_SMOKE4", "S_SMOKE5",
    "S_TRACER", "S_TRACER2", "S_TRACEEXP1", "S_TRACEEXP2", "S_TRACEEXP3",
    "S_SKEL_STND", "S_SKEL_STND2", "S_SKEL_RUN1", "S_SKEL_RUN2", "S_SKEL_RUN3",
    "S_SKEL_RUN4", "S_SKEL_RUN5", "S_SKEL_RUN6", "S_SKEL_RUN7", "S_SKEL_RUN8",
    "S_SKEL_RUN9", "S_SKEL_RUN10", "S_SKEL_RUN11", "S_SKEL_RUN12", "S_SKEL_FIST1",
    "S_SKEL_FIST2", "S_SKEL_FIST3", "S_SKEL_FIST4", "S_SKEL_MISS1", "S_SKEL_MISS2",
    "S_SKEL_MISS3", "S_SKEL_MISS4", "S_SKEL_PAIN", "S_SKEL_PAIN2", "S_SKEL_DIE1",
    "S_SKEL_DIE2", "S_SKEL_DIE3", "S_SKEL_DIE4", "S_SKEL_DIE5", "S_SKEL_DIE6",
    "S_SKEL_RAISE1", "S_SKEL_RAISE2", "S_SKEL_RAISE3", "S_SKEL_RAISE4",
    "S_SKEL_RAISE5", "S_SKEL_RAISE6",
    "S_FATSHOT1", "S_FATSHOT2", "S_FATSHOTX1", "S_FATSHOTX2", "S_FATSHOTX3",
    "S_FATT_STND", "S_FATT_STND2", "S_FATT_RUN1", "S_FATT_RUN2", "S_FATT_RUN3",
    "S_FATT_RUN4", "S_FATT_RUN5", "S_FATT_RUN6", "S_FATT_RUN7", "S_FATT_RUN8",
    "S_FATT_RUN9", "S_FATT_RUN10", "S_FATT_RUN11", "S_FATT_RUN12", "S_FATT_ATK1",
    "S_FATT_ATK2", "S_FATT_ATK3", "S_FATT_ATK4", "S_FATT_ATK5", "S_FATT_ATK6",
    "S_FATT_ATK7", "S_FATT_ATK8", "S_FATT_ATK9", "S_FATT_ATK10", "S_FATT_PAIN",
    "S_FATT_PAIN2", "S_FATT_DIE1", "S_FATT_DIE2", "S_FATT_DIE3", "S_FATT_DIE4",
    "S_FATT_DIE5", "S_FATT_DIE6", "S_FATT_DIE7", "S_FATT_DIE8", "S_FATT_DIE9",
    "S_FATT_DIE10", "S_FATT_RAISE1", "S_FATT_RAISE2", "S_FATT_RAISE3", "S_FATT_RAISE4",
    "S_FATT_RAISE5", "S_FATT_RAISE6", "S_FATT_RAISE7", "S_FATT_RAISE8",
    "S_CPOS_STND", "S_CPOS_STND2", "S_CPOS_RUN1", "S_CPOS_RUN2", "S_CPOS_RUN3",
    "S_CPOS_RUN4", "S_CPOS_RUN5", "S_CPOS_RUN6", "S_CPOS_RUN7", "S_CPOS_RUN8",
    "S_CPOS_ATK1", "S_CPOS_ATK2", "S_CPOS_ATK3", "S_CPOS_ATK4", "S_CPOS_PAIN",
    "S_CPOS_PAIN2", "S_CPOS_DIE1", "S_CPOS_DIE2", "S_CPOS_DIE3", "S_CPOS_DIE4",
    "S_CPOS_DIE5", "S_CPOS_DIE6", "S_CPOS_DIE7", "S_CPOS_XDIE1", "S_CPOS_XDIE2",
    "S_CPOS_XDIE3", "S_CPOS_XDIE4", "S_CPOS_XDIE5", "S_CPOS_XDIE6", "S_CPOS_RAISE1",
    "S_CPOS_RAISE2", "S_CPOS_RAISE3", "S_CPOS_RAISE4", "S_CPOS_RAISE5",
    "S_CPOS_RAISE6", "S_CPOS_RAISE7",
    "S_TROO_STND", "S_TROO_STND2", "S_TROO_RUN1", "S_TROO_RUN2", "S_TROO_RUN3",
    "S_TROO_RUN4", "S_TROO_RUN5", "S_TROO_RUN6", "S_TROO_RUN7", "S_TROO_RUN8",
    "S_TROO_ATK1", "S_TROO_ATK2", "S_TROO_ATK3", "S_TROO_PAIN", "S_TROO_PAIN2",
    "S_TROO_DIE1", "S_TROO_DIE2", "S_TROO_DIE3", "S_TROO_DIE4", "S_TROO_DIE5",
    "S_TROO_XDIE1", "S_TROO_XDIE2", "S_TROO_XDIE3", "S_TROO_XDIE4", "S_TROO_XDIE5",
    "S_TROO_XDIE6", "S_TROO_XDIE7", "S_TROO_XDIE8", "S_TROO_RAISE1",
    "S_TROO_RAISE2", "S_TROO_RAISE3", "S_TROO_RAISE4", "S_TROO_RAISE5",
    "S_SARG_STND", "S_SARG_STND2", "S_SARG_RUN1", "S_SARG_RUN2", "S_SARG_RUN3",
    "S_SARG_RUN4", "S_SARG_RUN5", "S_SARG_RUN6", "S_SARG_RUN7", "S_SARG_RUN8",
    "S_SARG_ATK1", "S_SARG_ATK2", "S_SARG_ATK3", "S_SARG_PAIN", "S_SARG_PAIN2",
    "S_SARG_DIE1", "S_SARG_DIE2", "S_SARG_DIE3", "S_SARG_DIE4", "S_SARG_DIE5",
    "S_SARG_DIE6", "S_SARG_RAISE1", "S_SARG_RAISE2", "S_SARG_RAISE3",
    "S_SARG_RAISE4", "S_SARG_RAISE5", "S_SARG_RAISE6",
    "S_HEAD_STND", "S_HEAD_RUN1", "S_HEAD_ATK1", "S_HEAD_ATK2", "S_HEAD_ATK3",
    "S_HEAD_PAIN", "S_HEAD_PAIN2", "S_HEAD_PAIN3", "S_HEAD_DIE1", "S_HEAD_DIE2",
    "S_HEAD_DIE3", "S_HEAD_DIE4", "S_HEAD_DIE5", "S_HEAD_DIE6", "S_HEAD_RAISE1",
    "S_HEAD_RAISE2", "S_HEAD_RAISE3", "S_HEAD_RAISE4", "S_HEAD_RAISE5", "S_HEAD_RAISE6",
    "S_BRBALL1", "S_BRBALL2", "S_BRBALLX1", "S_BRBALLX2", "S_BRBALLX3",
    "S_BOSS_STND", "S_BOSS_STND2", "S_BOSS_RUN1", "S_BOSS_RUN2", "S_BOSS_RUN3",
    "S_BOSS_RUN4", "S_BOSS_RUN5", "S_BOSS_RUN6", "S_BOSS_RUN7", "S_BOSS_RUN8",
    "S_BOSS_ATK1", "S_BOSS_ATK2", "S_BOSS_ATK3", "S_BOSS_PAIN", "S_BOSS_PAIN2",
    "S_BOSS_DIE1", "S_BOSS_DIE2", "S_BOSS_DIE3", "S_BOSS_DIE4", "S_BOSS_DIE5",
    "S_BOSS_DIE6", "S_BOSS_DIE7", "S_BOSS_RAISE1", "S_BOSS_RAISE2", "S_BOSS_RAISE3",
    "S_BOSS_RAISE4", "S_BOSS_RAISE5", "S_BOSS_RAISE6", "S_BOSS_RAISE7",
    "S_BOS2_STND", "S_BOS2_STND2", "S_BOS2_RUN1", "S_BOS2_RUN2", "S_BOS2_RUN3",
    "S_BOS2_RUN4", "S_BOS2_RUN5", "S_BOS2_RUN6", "S_BOS2_RUN7", "S_BOS2_RUN8",
    "S_BOS2_ATK1", "S_BOS2_ATK2", "S_BOS2_ATK3", "S_BOS2_PAIN", "S_BOS2_PAIN2",
    "S_BOS2_DIE1", "S_BOS2_DIE2", "S_BOS2_DIE3", "S_BOS2_DIE4", "S_BOS2_DIE5",
    "S_BOS2_DIE6", "S_BOS2_DIE7", "S_BOS2_RAISE1", "S_BOS2_RAISE2", "S_BOS2_RAISE3",
    "S_BOS2_RAISE4", "S_BOS2_RAISE5", "S_BOS2_RAISE6", "S_BOS2_RAISE7",
    "S_SKULL_STND", "S_SKULL_STND2", "S_SKULL_RUN1", "S_SKULL_RUN2",
    "S_SKULL_ATK1", "S_SKULL_ATK2", "S_SKULL_ATK3", "S_SKULL_ATK4",
    "S_SKULL_PAIN", "S_SKULL_PAIN2", "S_SKULL_DIE1", "S_SKULL_DIE2",
    "S_SKULL_DIE3", "S_SKULL_DIE4", "S_SKULL_DIE5", "S_SKULL_DIE6",
    "S_SPID_STND", "S_SPID_STND2", "S_SPID_RUN1", "S_SPID_RUN2", "S_SPID_RUN3",
    "S_SPID_RUN4", "S_SPID_RUN5", "S_SPID_RUN6", "S_SPID_RUN7", "S_SPID_RUN8",
    "S_SPID_RUN9", "S_SPID_RUN10", "S_SPID_RUN11", "S_SPID_RUN12",
    "S_SPID_ATK1", "S_SPID_ATK2", "S_SPID_ATK3", "S_SPID_ATK4",
    "S_SPID_PAIN", "S_SPID_PAIN2", "S_SPID_DIE1", "S_SPID_DIE2", "S_SPID_DIE3",
    "S_SPID_DIE4", "S_SPID_DIE5", "S_SPID_DIE6", "S_SPID_DIE7", "S_SPID_DIE8",
    "S_SPID_DIE9", "S_SPID_DIE10", "S_SPID_DIE11",
    "S_BSPI_STND", "S_BSPI_STND2", "S_BSPI_SIGHT", "S_BSPI_RUN1", "S_BSPI_RUN2",
    "S_BSPI_RUN3", "S_BSPI_RUN4", "S_BSPI_RUN5", "S_BSPI_RUN6", "S_BSPI_RUN7",
    "S_BSPI_RUN8", "S_BSPI_RUN9", "S_BSPI_RUN10", "S_BSPI_RUN11", "S_BSPI_RUN12",
    "S_BSPI_ATK1", "S_BSPI_ATK2", "S_BSPI_ATK3", "S_BSPI_ATK4", "S_BSPI_PAIN",
    "S_BSPI_PAIN2", "S_BSPI_DIE1", "S_BSPI_DIE2", "S_BSPI_DIE3", "S_BSPI_DIE4",
    "S_BSPI_DIE5", "S_BSPI_DIE6", "S_BSPI_DIE7", "S_BSPI_RAISE1", "S_BSPI_RAISE2",
    "S_BSPI_RAISE3", "S_BSPI_RAISE4", "S_BSPI_RAISE5", "S_BSPI_RAISE6", "S_BSPI_RAISE7",
    "S_ARACH_PLAZ", "S_ARACH_PLAZ2", "S_ARACH_PLEX", "S_ARACH_PLEX2", "S_ARACH_PLEX3",
    "S_ARACH_PLEX4", "S_ARACH_PLEX5",
    "S_CYBER_STND", "S_CYBER_STND2", "S_CYBER_RUN1", "S_CYBER_RUN2", "S_CYBER_RUN3",
    "S_CYBER_RUN4", "S_CYBER_RUN5", "S_CYBER_RUN6", "S_CYBER_RUN7", "S_CYBER_RUN8",
    "S_CYBER_ATK1", "S_CYBER_ATK2", "S_CYBER_ATK3", "S_CYBER_ATK4", "S_CYBER_ATK5",
    "S_CYBER_ATK6", "S_CYBER_PAIN", "S_CYBER_DIE1", "S_CYBER_DIE2", "S_CYBER_DIE3",
    "S_CYBER_DIE4", "S_CYBER_DIE5", "S_CYBER_DIE6", "S_CYBER_DIE7", "S_CYBER_DIE8",
    "S_CYBER_DIE9", "S_CYBER_DIE10",
	"S_PAIN_STND",
    "S_PAIN_RUN1", "S_PAIN_RUN2", "S_PAIN_RUN3", "S_PAIN_RUN4", "S_PAIN_RUN5",
    "S_PAIN_RUN6", "S_PAIN_ATK1", "S_PAIN_ATK2", "S_PAIN_ATK3", "S_PAIN_ATK4",
    "S_PAIN_PAIN", "S_PAIN_PAIN2", "S_PAIN_DIE1", "S_PAIN_DIE2", "S_PAIN_DIE3",
    "S_PAIN_DIE4", "S_PAIN_DIE5", "S_PAIN_DIE6", "S_PAIN_RAISE1", "S_PAIN_RAISE2",
    "S_PAIN_RAISE3", "S_PAIN_RAISE4", "S_PAIN_RAISE5", "S_PAIN_RAISE6",
    "S_SSWV_STND", "S_SSWV_STND2", "S_SSWV_RUN1", "S_SSWV_RUN2", "S_SSWV_RUN3",
    "S_SSWV_RUN4", "S_SSWV_RUN5", "S_SSWV_RUN6", "S_SSWV_RUN7", "S_SSWV_RUN8",
    "S_SSWV_ATK1", "S_SSWV_ATK2", "S_SSWV_ATK3", "S_SSWV_ATK4", "S_SSWV_ATK5",
    "S_SSWV_ATK6", "S_SSWV_PAIN", "S_SSWV_PAIN2", "S_SSWV_DIE1", "S_SSWV_DIE2",
    "S_SSWV_DIE3", "S_SSWV_DIE4", "S_SSWV_DIE5", "S_SSWV_XDIE1", "S_SSWV_XDIE2",
    "S_SSWV_XDIE3", "S_SSWV_XDIE4", "S_SSWV_XDIE5", "S_SSWV_XDIE6", "S_SSWV_XDIE7",
    "S_SSWV_XDIE8", "S_SSWV_XDIE9", "S_SSWV_RAISE1", "S_SSWV_RAISE2", "S_SSWV_RAISE3",
    "S_SSWV_RAISE4", "S_SSWV_RAISE5",
    "S_KEENSTND", "S_COMMKEEN", "S_COMMKEEN2", "S_COMMKEEN3", "S_COMMKEEN4",
    "S_COMMKEEN5", "S_COMMKEEN6", "S_COMMKEEN7", "S_COMMKEEN8", "S_COMMKEEN9",
    "S_COMMKEEN10", "S_COMMKEEN11", "S_COMMKEEN12", "S_KEENPAIN", "S_KEENPAIN2",
    "S_BRAIN", "S_BRAIN_PAIN", "S_BRAIN_DIE1", "S_BRAIN_DIE2", "S_BRAIN_DIE3",
    "S_BRAIN_DIE4", "S_BRAINEYE", "S_BRAINEYESEE", "S_BRAINEYE1",
    "S_SPAWN1", "S_SPAWN2", "S_SPAWN3", "S_SPAWN4",
    "S_SPAWNFIRE1", "S_SPAWNFIRE2", "S_SPAWNFIRE3", "S_SPAWNFIRE4",
    "S_SPAWNFIRE5", "S_SPAWNFIRE6", "S_SPAWNFIRE7", "S_SPAWNFIRE8",
    "S_BRAINEXPLODE1", "S_BRAINEXPLODE2", "S_BRAINEXPLODE3",
    "S_ARM1", "S_ARM1A", "S_ARM2", "S_ARM2A",
    "S_BAR1", "S_BAR2", "S_BEXP", "S_BEXP2", "S_BEXP3",
    "S_BEXP4", "S_BEXP5", "S_BBAR1", "S_BBAR2", "S_BBAR3",
    "S_BON1", "S_BON1A", "S_BON1B", "S_BON1C", "S_BON1D", "S_BON1E",
    "S_BON2", "S_BON2A", "S_BON2B", "S_BON2C", "S_BON2D", "S_BON2E",
    "S_BKEY", "S_BKEY2", "S_RKEY", "S_RKEY2", "S_YKEY", "S_YKEY2",
    "S_BSKULL", "S_BSKULL2", "S_RSKULL", "S_RSKULL2", "S_YSKULL", "S_YSKULL2",
    "S_STIM", "S_MEDI",
    "S_SOUL", "S_SOUL2", "S_SOUL3", "S_SOUL4", "S_SOUL5", "S_SOUL6",
    "S_PINV", "S_PINV2", "S_PINV3", "S_PINV4",
    "S_PSTR",
    "S_PINS", "S_PINS2", "S_PINS3", "S_PINS4",
    "S_MEGA", "S_MEGA2", "S_MEGA3", "S_MEGA4",
    "S_SUIT",
    "S_PMAP", "S_PMAP2", "S_PMAP3", "S_PMAP4", "S_PMAP5", "S_PMAP6",
    "S_PVIS", "S_PVIS2",
    "S_CLIP", "S_AMMO", "S_ROCK", "S_BROK",
    "S_CELL", "S_CELP", "S_SHEL", "S_SBOX",
    "S_BPAK",
    "S_BFUG", "S_MGUN", "S_CSAW", "S_LAUN", "S_PLAS", "S_SHOT", "S_SHOT2",
    "S_COLU", "S_STALAG",
    "S_BLOODYTWITCH", "S_BLOODYTWITCH2",
    "S_BLOODYTWITCH3", "S_BLOODYTWITCH4",
    "S_DEADTORSO", "S_DEADBOTTOM",
    "S_HEADSONSTICK",
    "S_GIBS",
    "S_HEADONASTICK", "S_HEADCANDLES", "S_HEADCANDLES2", "S_DEADSTICK",
    "S_LIVESTICK", "S_LIVESTICK2",
    "S_MEAT2", "S_MEAT3", "S_MEAT4", "S_MEAT5",
    "S_STALAGTITE",
    "S_TALLGRNCOL", "S_SHRTGRNCOL", "S_TALLREDCOL", "S_SHRTREDCOL",
    "S_CANDLESTIK", "S_CANDELABRA", "S_SKULLCOL",
    "S_TORCHTREE", "S_BIGTREE", "S_TECHPILLAR",
    "S_EVILEYE", "S_EVILEYE2", "S_EVILEYE3", "S_EVILEYE4",
    "S_FLOATSKULL", "S_FLOATSKULL2", "S_FLOATSKULL3",
    "S_HEARTCOL", "S_HEARTCOL2",
    "S_BLUETORCH", "S_BLUETORCH2", "S_BLUETORCH3", "S_BLUETORCH4",
    "S_GREENTORCH", "S_GREENTORCH2", "S_GREENTORCH3", "S_GREENTORCH4",
    "S_REDTORCH", "S_REDTORCH2", "S_REDTORCH3", "S_REDTORCH4",
    "S_BTORCHSHRT", "S_BTORCHSHRT2", "S_BTORCHSHRT3", "S_BTORCHSHRT4",
    "S_GTORCHSHRT", "S_GTORCHSHRT2", "S_GTORCHSHRT3", "S_GTORCHSHRT4",
    "S_RTORCHSHRT", "S_RTORCHSHRT2", "S_RTORCHSHRT3", "S_RTORCHSHRT4",
    "S_HANGNOGUTS", "S_HANGBNOBRAIN", "S_HANGTLOOKDN",
    "S_HANGTSKULL", "S_HANGTLOOKUP", "S_HANGTNOBRAIN",
    "S_COLONGIBS", "S_SMALLPOOL", "S_BRAINSTEM",
    "S_TECHLAMP", "S_TECHLAMP2", "S_TECHLAMP3", "S_TECHLAMP4",
    "S_TECH2LAMP", "S_TECH2LAMP2", "S_TECH2LAMP3", "S_TECH2LAMP4",

    // BOOM and MBF:
    "S_TNT1",
    "S_GRENADE", "S_DETONATE", "S_DETONATE2", "S_DETONATE3",
    "S_DOGS_STND", "S_DOGS_STND2", "S_DOGS_RUN1", "S_DOGS_RUN2",
    "S_DOGS_RUN3", "S_DOGS_RUN4", "S_DOGS_RUN5", "S_DOGS_RUN6",
    "S_DOGS_RUN7", "S_DOGS_RUN8",
    "S_DOGS_ATK1", "S_DOGS_ATK2", "S_DOGS_ATK3",
    "S_DOGS_PAIN", "S_DOGS_PAIN2",
    "S_DOGS_DIE1", "S_DOGS_DIE2", "S_DOGS_DIE3", "S_DOGS_DIE4",
    "S_DOGS_DIE5", "S_DOGS_DIE6",
    "S_DOGS_RAISE1", "S_DOGS_RAISE2", "S_DOGS_RAISE3",
    "S_DOGS_RAISE4", "S_DOGS_RAISE5", "S_DOGS_RAISE6",
    "S_MUSHROOM"
};
#endif

}  // Deh_Edge
